package cn.estsh.i3plus.core.api.iservice.busi;

import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.model.platform.UserDetailModel;
import cn.estsh.i3plus.pojo.platform.bean.*;
import io.swagger.annotations.ApiOperation;

import java.util.List;

/**
 * @Description :人员业务服务接口
 * @Reference :
 * @Author : Adair Peng
 * @CreateDate : 2018-12-22 16:15
 * @Modify:
 **/
public interface IPersonnelService {

    /**
     * 保存账号信息
     * @param user 账号信息
     * @return 保存之后的账号信息
     */
    @ApiOperation(value = "保存账号")
    SysUser saveSysUser(SysUser user);

    /**
     * 保存用户信息
     * @param info 账号信息
     * @return 保存后的账号信息
     */
    @ApiOperation(value = "保存用户")
    SysUserInfo saveSysUserInfo(SysUserInfo info);

    @ApiOperation(value = "更新用户状态")
    void updateSysUserInfoStatus(Long userInfoId,Integer status);

    @ApiOperation(value = "修改用户主组织信息")
    void updateSysUserOrganize(Long userId, Long organizeId);

    @ApiOperation(value = "删除账号信息")
    void deleteSysUser(Long[] userIdList,String userName);

    @ApiOperation(value = "删除用户信息")
    void deleteSysUserInfo(Long[] infoIdList,String userName);

    @ApiOperation(value = "获取账号信息")
    SysUser getSysUserById(Long id);

    @ApiOperation(value = "查询账号ID",notes = "根据部门ID 查询所有的人员信息")
    List<Long> findSysUserIdByDepartmentId(Long ...departmentId);

//    @ApiOperation(value = "查询账号ID",notes = "根据组织代码 查询所有的人员信息")
//    List<Long> findSysUserIdByOrganizeCode(String ...organizeCode);

    @ApiOperation(value = "查询账号信息")
    List<SysUser> findSysUserByIds(Long[] ids);

    @ApiOperation(value = "查询账号信息")
    ListPager<SysUser> querySysUser(UserDetailModel model, Pager pager);

    @ApiOperation(value = "查询账号信息")
    List<SysUser> querySysUserByUserDetailModelOrIds(UserDetailModel model, List<Long> ids);

    @ApiOperation(value = "查询账号信息")
    ListPager<SysUser> querySysUserPagerByUserDetailModelOrIds(UserDetailModel model,Pager pager, List<Long> ids);

    @ApiOperation(value = "获取用户信息")
    SysUserInfo getSysUserInfoById(Long id);

    @ApiOperation(value = "查询账号ID",notes = "根据部门ID 查询所有的人员信息")
    List<Long> findSysUserInfoIdByDepartmentId(Long ...departmentId);

//    @ApiOperation(value = "查询账号ID",notes = "根据组织代码 查询所有的人员信息")
//    List<Long> findSysUserInfoIdByOrganizeCode(String ...organizeCode);
//
//    @ApiOperation(value = "获取部门")
//    SysDepartment getSysDepartmentById(Long id);

    @ApiOperation(value = "用户登录")
    SessionUser queryUserLogin(String userName, String password, String languageCode);

    @ApiOperation(value = "查询用户信息")
    ListPager<SysUserInfo> querySysUserInfo(UserDetailModel model, Pager pager);

    @ApiOperation(value = "查询用户信息")
    List<SysUserInfo> findSysUserInfoByIds(Long[] ids);

    @ApiOperation(value = "查询角色",notes = "查询制定用户的角色信息")
    List<SysRole> findSysRoleByUserId(Long userId);

//    @ApiOperation(value = "查询角色",notes = "查询所有角色信息")
//    List<SysRole> findSysRoleList();

    @ApiOperation(value = "查询权限",notes = "查询所有权限")
    List<SysMenu> findSysMenuList();

    @ApiOperation(value = "查询权限",notes = "查询所有权限")
    List<SysMenu> findSysMenuByUserId(Long userId);

//    @ApiOperation(value = "查询权限",notes = "查询所有权限")
//    List<SysMenu> findSysMenuByIdList(List<Long> ids);

    @ApiOperation(value = "查询权限",notes = "查询所有权限")
    List<SysMenu> findSysMenuByRefRoleMenu(List<SysRefRoleMenu> refList);

    @ApiOperation(value = "查询账号角色关系")
    List<SysRefUserRole> findSysRefUserRole(Long userId);

    @ApiOperation(value = "查询账号角色关系")
    List<SysRefUserRole> findSysRefUserRole(Long roleId,String organizeCode);

    @ApiOperation(value = "查询账号角色关系")
    List<SysRefUserRole> findSysRefUserRole(Long userId,Long roleId,String organizeCode);

    @ApiOperation(value = "角色权限关系",notes = "根据角色ID 查询权限")
    List<SysRefRoleMenu> findSysRefRoleMenuByUserId(Long userId);

    @ApiOperation(value = "角色权限关系",notes = "根据角色ID 查询权限")
    List<SysRefRoleMenu> findSysRefRoleMenuByRoleIds(List<Long> roleIdList);

    @ApiOperation(value = "角色权限关系",notes = "根据角色ID 查询权限")
    List<SysRefRoleMenu> findSysRefRoleMenuByRole(List<SysRole> roleList);

    @ApiOperation(value = "角色权限关系",notes = "根据角色ID 查询权限")
    List<SysRefRoleMenu> findSysRefRoleMenuByRefUserRole(List<SysRefUserRole> refList);

    @ApiOperation(value = "查询账号部门关系")
    List<SysRefUserDepartment> findSysRefUserDepartmentByUserId(Long userId);

    List<SysRefUserDepartment> findSysRefUserDepartmentByOrganizeIdList(List<Long> organizeIdList);

    List<SysRefUserDepartment> findSysRefUserDepartmentByOrganizeCode(String organizeCode);

    List<SysRefUserDepartment> findSysRefUserDepartmentByDepartmentIdList(List<Long> departmentIdList);

    @ApiOperation(value = "查询账号部门关系")
    List<SysRefUserDepartment> querySysRefUserDepartmentByUserDetailModel(UserDetailModel model);

    @ApiOperation(value = "查询用户部门关系")
    List<SysRefUserInfoDepartment> findSysRefUserInfoDepartmentById(Long infoId);

    List<SysRefUserInfoDepartment> findSysRefUserInfoDepartmentByOrganizeIdList(List<Long> organizeIdList);

    List<SysRefUserInfoDepartment> findSysRefUserInfoDepartmentByDepartmentIdList(List<Long> departmentIdList);

    List<SysDepartment> findSysDepartmentByOrganizeIdList(List<Long> organizeIdList);

    @ApiOperation(value = "查询用户部门关系")
    List<SysDepartment> findSysUserInfoDepartmentByInfoId(Long infoId);

    @ApiOperation(value = "查询账号部门关系")
    List<SysDepartment> findSysUserDepartmentByInfoId(Long infoId);

//    @ApiOperation(value = "查询所有部门信息")
//    List<SysDepartment> findSysDepartmentList();

    @ApiOperation(value = "查询用户部门关系")
    List<SysOrganize> findSysUserInfoOrganize(Long infoId);

    @ApiOperation(value = "查询账户组织关系")
    List<SysOrganize> findSysUserOrganize(Long infoId);

//    List<SysOrganize> findSysOrganizeList();
//
//    List<SysPosition> findSysPositionList();

    @ApiOperation(value = "查询用户岗位关系")
    List<SysRefUserPosition> findSysRefUserPositionByUserInfoId(Long infoId);

    List<SysRefUserPosition> findSysRefUserPositionByIdList(List<Long> idList);

    @ApiOperation(value = "账号唯一检查")
    void checkSysUserOnly(SysUser user);

    @ApiOperation(value = "重置密码检查")
    Integer checkSysUserResetPassword(SysUser user);

    @ApiOperation(value = "岗位关系检查")
    void checkSysPositionRef(Long positionId);

    @ApiOperation(value = "部门关系检查")
    void checkSysDepartmentRef(Long departmentId);

    @ApiOperation(value = "组织关系检查")
    void checkSysOrganizeRef(Long organizeId);

//    @ApiOperation(value = "密码规则校验")
//    void checkSysUserPassword(String password);

//    @ApiOperation(value = "岗位关系检查")
//    void refreshUpdateMenuRdd(Long menuId);

    @ApiOperation(value = "刷新账号部门")
    void refreshRefSysUserDepartment(Long userId, Long[] departmentIds);

    @ApiOperation(value = "刷新账号角色")
    void refreshRefSysUserRole(Long userId, Long[] roleIds);

//    @ApiOperation(value = "刷新账号部门")
//    void refreshUpdateRefSysUserDepartmentRdd(Long departmentId);

    @ApiOperation(value = "刷新用户部门")
    void refreshUpdateRefSysUserInfoDepartment(Long userInfoId, Long[] departmentIds);

    @ApiOperation(value = "刷新用户岗位")
    void refreshRefSysUserInfoPosition(Long userInfoId, Long[] positionIds);


//    /**
//     * 根据组织代码查询组织信息
//     * @return
//     */
//    @ApiOperation(value = "根据组织代码查询组织信息")
//    SysOrganize getSysOrganizeByCode(String code);
}
