package cn.estsh.i3plus.core.apiservice.controller.busi;

import cn.estsh.i3plus.core.api.iservice.busi.ISysDataSourceService;
import cn.estsh.i3plus.core.api.iservice.busi.ISysPojoVersionPlanService;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.util.CommonConstWords;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.enumutil.ResourceEnumUtil;
import cn.estsh.i3plus.pojo.platform.bean.SysDataSource;
import cn.estsh.i3plus.pojo.platform.bean.SysFile;
import cn.estsh.i3plus.pojo.platform.bean.SysPojoVersionPlan;
import cn.estsh.impp.framework.base.controller.CrudBaseController;
import cn.estsh.impp.framework.base.service.ICrudService;
import cn.estsh.impp.framework.boot.datasource.ImppJdbcTemplateFactory;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ResultBean;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.web.bind.annotation.*;

import java.sql.Connection;
import java.sql.SQLException;
import java.util.Objects;

/**
 * @Description :
 * @Reference :
 * @Author : wei.peng
 * @CreateDate : 20-4-21 下午2:56
 * @Modify:
 **/
@RestController
@Api(tags = "数据源服务")
@RequestMapping(CommonConstWords.BASE_URL_CORE+"/data-source")
public class SysDataSourceController extends CrudBaseController<SysDataSource> {
    @Autowired
    private ISysDataSourceService dataSourceService;

    @Override
    public ICrudService getCrudService() {
        return dataSourceService;
    }

    @Override
    public SysDataSource validatorInsertBean(SysDataSource bean) throws Exception {
        ValidatorBean.beginValid(bean)
                .notNull("sourceName",bean.getSourceName())
                .notNull("softType",bean.getSoftType())
                .notNull("sourceCode",bean.getSourceCode())
                .notNull("sourceType",bean.getSourceType())
                .notNull("sourceHost",bean.getSourceHost())
                .notNull("sourcePort",bean.getSourcePort())
                .notNull("sourceUserName",bean.getSourceUserName())
                .notNull("sourcePassword",bean.getSourcePassword());

        try {
            bean.setSourceStatus(CommonEnumUtil.DATA_SOURCE_STATUS.CONN_FAILURE.getValue());
            NamedParameterJdbcTemplate template = ImppJdbcTemplateFactory.getNamedParameterJdbcTemplate(bean);
            if(template != null) {
                try {
                    Connection conn = template.getJdbcTemplate().getDataSource().getConnection();
                    if(Objects.nonNull(conn)){
                        bean.setSourceStatus(CommonEnumUtil.DATA_SOURCE_STATUS.CONN_SUCCESS.getValue());
                    }
                }catch (SQLException e){
                    e.printStackTrace();
                }
            }
        }catch (Exception e){
            e.printStackTrace();
        }

        return bean;
    }

    @GetMapping(value = "/connection-check/{id}")
    @ApiOperation(value = "数据源连接检查")
    public ResultBean<SysDataSource> checkConnection(@PathVariable("id") Long id) {
        try {
            SysDataSource source = dataSourceService.get(id);
            NamedParameterJdbcTemplate template = ImppJdbcTemplateFactory.getNamedParameterJdbcTemplate(source);
            if(template != null) {
                try {
                    Connection conn = template.getJdbcTemplate().getDataSource().getConnection();
                    if(Objects.nonNull(conn)){
                        source.setSourceStatus(CommonEnumUtil.DATA_SOURCE_STATUS.CONN_SUCCESS.getValue());
                        dataSourceService.update(source);

                        return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
                    }
                }catch (SQLException e){
                    e.printStackTrace();
                }
            }
            return ResultBean.fail("操作失败").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }
}
