package cn.estsh.i3plus.core.apiservice.controller.busi;

import cn.estsh.i3plus.platform.common.util.PlatformConstWords;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.enumutil.ImppEnumUtil;
import cn.estsh.impp.framework.base.controller.CoreBaseController;
import cn.estsh.impp.framework.boot.util.LocaleUtils;
import cn.estsh.impp.framework.boot.util.ResultBean;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.List;

/**
 * @Description : 系统对外枚举服务
 * @Reference :
 * @Author : wei.peng
 * @Date : 2018-10-22 15:58
 * @Modify :
 **/
@RestController
@Api(tags = "系统枚举服务")
@RequestMapping(PlatformConstWords.BASE_URL + "/sys-enum")
public class SysEnumController extends CoreBaseController {

    /**
     * 功能类型
     *
     * @return 处理结果
     */
    @GetMapping("/method-level")
    @ApiOperation(value = "功能类型", notes = "查询全部功能类型功能")
    public ResultBean getEnumMethodLevel() {
        return new ResultBean(true, "操作成功",
                LocaleUtils.getEnumLocaleResValuesToList(CommonEnumUtil.METHOD_LEVEL.values()));
    }

    /**
     * 数据功能状态
     *
     * @return 处理结果
     */
    @GetMapping("/user-data-status")
    @ApiOperation(value = "数据功能状态", notes = "查询全部数据状态功能")
    public ResultBean getEnumUserDataStatus() {
        return new ResultBean(true, "操作成功",
                LocaleUtils.getEnumLocaleResValuesToList((CommonEnumUtil.DATA_STATUS.values())));
    }

    /**
     * 数据功能状态
     *
     * @return 处理结果
     */
    @GetMapping("/data-status")
    @ApiOperation(value = "数据功能状态", notes = "查询全部数据状态功能")
    public ResultBean getEnumDataStatus() {
        List<CommonEnumUtil.DATA_STATUS> enumList = new ArrayList();
        enumList.add(CommonEnumUtil.DATA_STATUS.ENABLE);
        enumList.add(CommonEnumUtil.DATA_STATUS.DISABLE);
        return new ResultBean(true, "操作成功",  LocaleUtils.getEnumLocaleResValueList(enumList));
    }

    /**
     * 组织类型
     *
     * @return 处理结果
     */
    @GetMapping("/organize-type")
    @ApiOperation(value = "组织类型", notes = "查询全部组织类型功能")
    public ResultBean getEnumOrganizeType() {
        return new ResultBean(true, "操作成功",
                LocaleUtils.getEnumLocaleResValuesToList(ImppEnumUtil.ORGANIZE_YTPE.values()));
    }

    /**
     * 任务类型
     *
     * @return 处理结果
     */
    @GetMapping("/task-method-type")
    @ApiOperation(value = "任务类型", notes = "查询全部任务类型功能")
    public ResultBean getTaskMethodType() {
        return new ResultBean(true, "操作成功",
                LocaleUtils.getEnumLocaleResValuesToList(ImppEnumUtil.TASK_METHOD_TYPE.values()));
    }

    /**
     * 用户类型
     *
     * @return 处理结果
     */
    @GetMapping("/user-type")
    @ApiOperation(value = "用户类型", notes = "查询全部用户类型功能")
    public ResultBean getUserType() {
        return new ResultBean(true, "操作成功",
                LocaleUtils.getEnumLocaleResValuesToList(CommonEnumUtil.USER_TYPE.values()));
    }

    /**
     * 用户账号状态
     *
     * @return 处理结果
     */
    @GetMapping("/user-status")
    @ApiOperation(value = "用户账号状态", notes = "查询全部用户账号功能")
    public ResultBean getUserStatus() {
        return new ResultBean(true, "操作成功",
                LocaleUtils.getEnumLocaleResValuesToList(CommonEnumUtil.USER_STATUS.values()));
    }

    /**
     * 日志等级
     *
     * @return 处理结果
     */
    @GetMapping("/log-level")
    @ApiOperation(value = "日志等级", notes = "日志等级")
    public ResultBean getLogLevel() {
        return new ResultBean(true, "操作成功",
                LocaleUtils.getEnumLocaleResValuesToList(ImppEnumUtil.LOG_LEVEL.values()));
    }

    /**
     * 软件类型
     *
     * @return 处理结果
     */
    @GetMapping("/soft-type")
    @ApiOperation(value = "软件类型", notes = "软件类型")
    public ResultBean getSoftType() {
        long startTime = System.currentTimeMillis();
        CommonEnumUtil.SOFT_TYPE[] softTypes = CommonEnumUtil.SOFT_TYPE.values();
        long enumTime = System.currentTimeMillis();
        System.out.println("耗时：" + (enumTime - startTime));
        List locale = LocaleUtils.getEnumLocaleResValuesToList(softTypes);
        System.out.println("耗时：" + (System.currentTimeMillis() - enumTime));
        return new ResultBean(true, "操作成功", locale);
    }

    /**
     * 操作日志类型
     *
     * @return 处理结果
     */
    @GetMapping("/operate-type")
    @ApiOperation(value = "操作日志类型", notes = "操作日志类型")
    public ResultBean getOperateType() {
        return new ResultBean(true, "操作成功",
                LocaleUtils.getEnumLocaleResValuesToList(ImppEnumUtil.OPERATE_TYPE.values()));
    }

    /**
     * @return 处理结果
     */
    @GetMapping("/user-message-status")
    @ApiOperation(value = "用户消息状态", notes = "用户消息状态")
    public ResultBean getUserMessageStatus() {
        return new ResultBean(true, "操作成功",
                LocaleUtils.getEnumLocaleResValuesToList(ImppEnumUtil.MESSAGE_STATUS.values()));
    }


    /**
     * @return 处理结果
     */
    @GetMapping("/user-message-type")
    @ApiOperation(value = "用户消息类型", notes = "用户消息类型")
    public ResultBean getUserMessageType() {
        return new ResultBean(true, "操作成功",
                LocaleUtils.getEnumLocaleResValuesToList(ImppEnumUtil.MESSAGE_TYPE.values()));
    }


    /**
     * @return 处理结果
     */
    @GetMapping("/tool-conn-type")
    @ApiOperation(value = "硬件连接类型", notes = "硬件连接类型")
    public ResultBean getToolConnType() {
        return new ResultBean(true, "操作成功",
                LocaleUtils.getEnumLocaleResValuesToList(ImppEnumUtil.TOOL_CONN_TYPE.values()));
    }


    /**
     * @return 处理结果
     */
    @GetMapping("/tool-data-type")
    @ApiOperation(value = "硬件数据类型", notes = "硬件数据类型")
    public ResultBean getToolDataType() {
        return new ResultBean(true, "操作成功",
                LocaleUtils.getEnumLocaleResValuesToList(ImppEnumUtil.TOOL_DATA_TYPE.values()));
    }


    /**
     * @return 处理结果
     */
    @GetMapping("/sys-config-type")
    @ApiOperation(value = "系统参数类型", notes = "系统参数")
    public ResultBean getSysParamType() {
        return new ResultBean(true, "操作成功",
                LocaleUtils.getEnumLocaleResValuesToList(ImppEnumUtil.SYS_CONFIG_TYPE.values()));
    }


    /**
     * @return 处理结果
     */
    @GetMapping("/sys-locale-resource-type")
    @ApiOperation(value = "系统资源类型")
    public ResultBean getSysLocaleResourceType() {
        return new ResultBean(true, "操作成功",
                LocaleUtils.getEnumLocaleResValuesToList(CommonEnumUtil.SYS_LOCALE_RESOURCE_TYPE.values()));
    }


    /**
     * @return 处理结果
     */
    @GetMapping("/user-sex")
    @ApiOperation(value = "用户性别", notes = "用户性别")
    public ResultBean getUserSex() {
        return new ResultBean(true, "操作成功",
                LocaleUtils.getEnumLocaleResValuesToList(CommonEnumUtil.USER_ESX.values()));
    }


    /**
     * @return 处理结果
     */
    @GetMapping("/user-info-status")
    @ApiOperation(value = "用户状态", notes = "用户状态")
    public ResultBean getUserInfoStatus() {
        return new ResultBean(true, "操作成功",
                LocaleUtils.getEnumLocaleResValuesToList(CommonEnumUtil.USER_INFO_STATUS.values()));
    }

    /**
     * @return 处理结果
     */
    @GetMapping("/dictionary-value-type")
    @ApiOperation(value = "字典参数值类型", notes = "字典参数值类型")
    public ResultBean getSysDictionaryValueType() {
        List<ImppEnumUtil.SYS_VALUE_TYPE> list = new ArrayList<>();
        list.add(ImppEnumUtil.SYS_VALUE_TYPE.STRING);
        list.add(ImppEnumUtil.SYS_VALUE_TYPE.NUMBER);
        list.add(ImppEnumUtil.SYS_VALUE_TYPE.DATE);
        list.add(ImppEnumUtil.SYS_VALUE_TYPE.SWITCH);

        return new ResultBean(true, "操作成功", LocaleUtils.getEnumLocaleResValueList(list));
    }

    /**
     * @return 处理结果
     */
    @GetMapping("/sys-value-type")
    @ApiOperation(value = "系统参数值类型", notes = "用户状态")
    public ResultBean getSysConfigValueType() {
        return new ResultBean(true, "操作成功",
                LocaleUtils.getEnumLocaleResValuesToList(ImppEnumUtil.SYS_VALUE_TYPE.values()));
    }



    /**
     * @return 处理结果
     */
    @GetMapping("/sys-config-group")
    @ApiOperation(value = "系统配置组", notes = "系统配置组")
    public ResultBean getSysConfigGroup() {
        return new ResultBean(true, "操作成功",
                LocaleUtils.getEnumLocaleResValuesToList(ImppEnumUtil.SYS_CONFIG_GROUP.values()));
    }

    /**
     * @return 处理结果
     */
    @GetMapping("/internal-notice-method")
    @ApiOperation(value = "内部通知方式", notes = "内部通知方式")
    public ResultBean getInternalNoticeMethod() {
        List<ImppEnumUtil.MESSAGE_TYPE> list = new ArrayList<>();
        list.add(ImppEnumUtil.MESSAGE_TYPE.MAIL);
        list.add(ImppEnumUtil.MESSAGE_TYPE.LETTER);
        return new ResultBean(true, "操作成功",  LocaleUtils.getEnumLocaleResValueList(list));
    }

    /**
     * 外部通知方式
     * @return 处理结果
     */
    @GetMapping("/external-notice-method")
    @ApiOperation(value = "外部通知方式", notes = "外部通知方式")
    public ResultBean getExternalNoticeMethod() {
        List<ImppEnumUtil.MESSAGE_TYPE> list = new ArrayList<>();
        list.add(ImppEnumUtil.MESSAGE_TYPE.MAIL);
        return new ResultBean(true, "操作成功",  LocaleUtils.getEnumLocaleResValueList(list));
    }

    /**
     * 字典分组
     * @return 处理结果
     */
    @GetMapping("/dictionary-group")
    @ApiOperation(value = "字典分组", notes = "字典分组")
    public ResultBean getDictionaryGroup() {
        return new ResultBean(true, "操作成功",
                LocaleUtils.getEnumLocaleResValuesToList(ImppEnumUtil.DICTIONARY_GROUP.values()));
    }

    @GetMapping("/log-storage-type")
    @ApiOperation(value = "日志存储类型")
    public ResultBean getLogStorageType() {
        return new ResultBean(true, "操作成功",
                LocaleUtils.getEnumLocaleResValuesToList(ImppEnumUtil.LOG_STORAGE.values()));
    }

    /**
     * @return 处理结果
     */
    @GetMapping("/log-print-type")
    @ApiOperation(value = "系统参数值类型", notes = "用户状态")
    public ResultBean getLogPrintType() {
        return new ResultBean(true, "操作成功",
                LocaleUtils.getEnumLocaleResValuesToList(CommonEnumUtil.LOG_DETAIL_SHOW_TYPE.values()));
    }

    @GetMapping("/data-source-type")
    @ApiOperation(value = "数据源类型", notes = "数据源类型")
    public ResultBean getDataSourceType() {
        return new ResultBean(true, "操作成功",
                LocaleUtils.getEnumLocaleResValuesToList(CommonEnumUtil.DATA_SOURCE_TYPE.values()));
    }

    @GetMapping("/message-send-status")
    @ApiOperation(value = "消息发送状态", notes = "消息发送状态")
    public ResultBean getMessageSendStatus() {
        return new ResultBean(true, "操作成功",
                LocaleUtils.getEnumLocaleResValuesToList(ImppEnumUtil.MESSAGE_SEND_STATUS.values()));
    }


}
