package cn.estsh.i3plus.core.apiservice.controller.busi;

import cn.estsh.i3plus.core.api.iservice.busi.ISysPojoVersionDetailService;
import cn.estsh.i3plus.platform.common.util.CommonConstWords;
import cn.estsh.i3plus.pojo.base.enumutil.ResourceEnumUtil;
import cn.estsh.i3plus.pojo.platform.bean.SysPojoVersionDetail;
import cn.estsh.impp.framework.base.controller.CrudBaseController;
import cn.estsh.impp.framework.base.service.ICrudService;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ResultBean;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * @Description :
 * @Reference :
 * @Author : wei.peng
 * @CreateDate : 20-4-21 下午2:56
 * @Modify:
 **/
@RestController
@Api(tags = "对象历史详情")
@RequestMapping(CommonConstWords.BASE_URL_CORE+"/pojo-version-detail")
public class SysPojoVersionDetailController extends CrudBaseController<SysPojoVersionDetail> {
    @Autowired
    private ISysPojoVersionDetailService pojoVersionService;

    @Override
    public ICrudService getCrudService() {
        return pojoVersionService;
    }

    /**
     * 查询对象历史计划
     *
     * @param pojoVersionId  对象版本id
     * @return 处理结果
     */
    @GetMapping(value = "/get-pojo-version-id/{pojoVersionId}")
    @ApiOperation(value = "查询对象历史计划", notes = "根据所属模块与className查询对象历史计划状态")
    public ResultBean getPojoVersionDetailByPojoVersionId(@PathVariable("pojoVersionId") Long pojoVersionId) {
        try {
            // 数据校验
            ValidatorBean.checkNotZero(pojoVersionId, "对象版本id 不能为空");
            SysPojoVersionDetail pojoVersionDetail = pojoVersionService.getPojoVersionDetailByPojoVersionId(pojoVersionId);
            return ResultBean.success("操作成功")
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode())
                    .setResultObject(pojoVersionDetail);
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }
}
