package cn.estsh.i3plus.core.apiservice.controller.busi;

import cn.estsh.i3plus.core.api.iservice.busi.ISysPojoVersionPlanService;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.util.CommonConstWords;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.enumutil.ResourceEnumUtil;
import cn.estsh.i3plus.pojo.platform.bean.SysPojoVersionPlan;
import cn.estsh.impp.framework.base.controller.CrudBaseController;
import cn.estsh.impp.framework.base.service.ICrudService;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ResultBean;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * @Description :
 * @Reference :
 * @Author : wei.peng
 * @CreateDate : 20-4-21 下午2:56
 * @Modify:
 **/
@RestController
@Api(tags = "对象历史计划")
@RequestMapping(CommonConstWords.BASE_URL_CORE + "/pojo-version-plan")
public class SysPojoVersionPlanController extends CrudBaseController<SysPojoVersionPlan> {
    @Autowired
    private ISysPojoVersionPlanService pojoVersionService;

    @Override
    public ICrudService getCrudService() {
        return pojoVersionService;
    }

    @Override
    public SysPojoVersionPlan validatorInsertBean(SysPojoVersionPlan bean) throws Exception {
        ValidatorBean.beginValid(bean)
                .notNull("refClass", bean.getRefClass())
                .notNull("softType", bean.getSoftType())
                .notNull("refClassTitle", bean.getRefClassTitle());
        bean.setPlanStatus(CommonEnumUtil.DATA_STATUS.ENABLE.getValue());

        pojoVersionService.checkSysPojoVersionPlanOnly(bean);
        return bean;
    }

    @Override
    public SysPojoVersionPlan validatorUpdateBean(SysPojoVersionPlan bean) throws Exception {
        ValidatorBean.beginValid(bean)
                .notNull("id", bean.getId())
                .notNull("refClass", bean.getRefClass())
                .notNull("softType", bean.getSoftType())
                .notNull("planStatus", bean.getPlanStatus())
                .notNull("refClassTitle", bean.getRefClassTitle());

        pojoVersionService.checkSysPojoVersionPlanOnly(bean);
        return bean;
    }

    /**
     * 修改角色状态
     *
     * @param id     角色id
     * @param status 状态
     * @return 处理结果
     */
    @PutMapping(value = "/status")
    @ApiOperation(value = "对象历史计划状态", notes = "根据ID修改对象历史计划状态")
    public ResultBean updateStatus(Long id, int status) {
        try {
            // 数据校验
            ValidatorBean.checkNotNull(id, "对象历史计划id 不能为空");
            ValidatorBean.checkNotZero(status, "对象历史计划状态不能为空");
            SysPojoVersionPlan plan = pojoVersionService.get(id);
            ValidatorBean.checkNotNull(plan, "不存在对象历史计划信息");

            plan.setPlanStatus(status);
            ConvertBean.modelUpdate(plan, getSessionUser());

            return ResultBean.success("操作成功")
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode())
                    .setResultObject(pojoVersionService.update(plan));
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 根据所属模块与className查询对象历史计划状态
     *
     * @param softType  角色id
     * @param className 状态
     * @return 处理结果
     */
    @GetMapping(value = "/get-soft-class")
    @ApiOperation(value = "查询对象历史计划", notes = "根据所属模块与className查询对象历史计划状态")
    public ResultBean getSysPojoVersionPlanBySoftTypeAndClassName(Integer softType, String className) {
        try {
            // 数据校验
            ValidatorBean.checkNotZero(softType, "所属模块 不能为空");
            ValidatorBean.checkNotNull(className, "关联类名不能为空 不能为空");
            SysPojoVersionPlan plan = pojoVersionService.getSysPojoVersionPlanBySoftTypeAndClassName(softType, className);
            ValidatorBean.checkNotNull(plan, "不存在对象历史计划信息");
            return ResultBean.success("操作成功")
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode())
                    .setResultObject(plan);
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

}
