package cn.estsh.i3plus.core.apiservice.controller.busi;

import cn.estsh.i3plus.core.api.iservice.busi.*;
import cn.estsh.i3plus.icloud.andon.sdk.IAndonScheduleJobCloud;
import cn.estsh.i3plus.icloud.mes.pcn.sdk.IMesPcnScheduleJobCloud;
import cn.estsh.i3plus.icloud.mes.sdk.IMesScheduleJobCloud;
import cn.estsh.i3plus.icloud.wms.sdk.IWmsScheduleJobCloud;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.platform.common.tool.ExcelTool;
import cn.estsh.i3plus.platform.common.tool.FileTool;
import cn.estsh.i3plus.platform.common.tool.StringTool;
import cn.estsh.i3plus.platform.common.util.PlatformConstWords;
import cn.estsh.i3plus.pojo.base.bean.BaseResultBean;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.enumutil.ResourceEnumUtil;
import cn.estsh.i3plus.pojo.platform.bean.SysRefTaskCyclePlan;
import cn.estsh.i3plus.pojo.platform.bean.SysTask;
import cn.estsh.i3plus.pojo.platform.bean.SysTaskCycle;
import cn.estsh.i3plus.pojo.platform.bean.SysTaskPlan;
import cn.estsh.impp.framework.base.controller.CoreBaseController;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.fileservice.ImppFileService;
import cn.estsh.impp.framework.boot.quartz.IQuartzJobService;
import cn.estsh.impp.framework.boot.quartz.ScheduleTool;
import cn.estsh.impp.framework.boot.util.RedisCacheTool;
import cn.estsh.impp.framework.boot.util.ResultBean;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.quartz.impl.triggers.CronTriggerImpl;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.persistence.EntityManager;
import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.net.URLEncoder;
import java.text.ParseException;
import java.util.List;
import java.util.Map;

/**
 * @Description : 任务周期管理服务
 * @Reference :
 * @Author : yunhao
 * @Date : 2018-12-27 15:54
 * @Modify :
 **/
@RestController
@Api(tags = "任务周期")
@RequestMapping(PlatformConstWords.BASE_URL + "/task-cycle")
public class SysTaskCycleController extends CoreBaseController {

    public static final Logger LOGGER = LoggerFactory.getLogger(SysTaskCycleController.class);

    @Autowired
    private ISysTaskCycleService sysTaskCycleService;

    @Autowired
    private ISysTaskPlanService sysTaskPlanService;

    @Autowired
    private ISysTaskService sysTaskService;

    @Autowired
    private ISysFileService sysFileService;

    @Autowired
    private ImppFileService fileService;

    @Autowired
    private ISysMessageService sysMessageService;

    @Autowired
    private EntityManager entityManager;


    /************ 微服定时任务 *********/
    @Autowired
    private ScheduleTool scheduleTool;

    @Autowired
    private IWmsScheduleJobCloud wmsScheduleJobCloud;

    @Autowired
    private IMesScheduleJobCloud mesScheduleJobCloud;

    @Autowired
    private IMesPcnScheduleJobCloud mesPcnScheduleJobCloud;

    @Autowired
    private IAndonScheduleJobCloud andonScheduleJobCloud;

    /**
     * 根据来源切换定时任务服务
     *
     * @param source CommonEnumUtil.SOFT_TYPE
     * @return 处理结果
     */
    private IQuartzJobService getJobService(int source) {
        IQuartzJobService quartzJobService = scheduleTool;

        //平台定时任务
        if (source == CommonEnumUtil.SOFT_TYPE.CORE.getValue()) {
            quartzJobService = scheduleTool;
            // WMS
        } else if (source == CommonEnumUtil.SOFT_TYPE.WMS.getValue()) {
            quartzJobService = wmsScheduleJobCloud;
            // MES
        } else if (source == CommonEnumUtil.SOFT_TYPE.MES.getValue()) {
            quartzJobService = mesScheduleJobCloud;
            // MES_PCN
        } else if (source == CommonEnumUtil.SOFT_TYPE.MES_PCN.getValue()) {
            quartzJobService = mesPcnScheduleJobCloud;
            // ANDON
        } else if (source == CommonEnumUtil.SOFT_TYPE.ANDON.getValue()) {
            quartzJobService = andonScheduleJobCloud;
        }
        return quartzJobService;
    }

    /**
     * 新增任务周期
     *
     * @param sysTaskCycle 任务周期
     * @return 处理结果
     */
    @PostMapping(value = "/insert")
    @ApiOperation(value = "新增任务周期", notes = "任务周期")
    public ResultBean insertSysTaskCycle(SysTaskCycle sysTaskCycle) {
        try {
            // 数据校验
            ValidatorBean.beginValid(sysTaskCycle)
                    .notNull("name", sysTaskCycle.getName())
                    .notNull("taskCycleExps", sysTaskCycle.getTaskCycleExps());

            CronTriggerImpl trigger = new CronTriggerImpl();
            trigger.setCronExpression(sysTaskCycle.getTaskCycleExps());
            trigger.computeFirstFireTime(null);

            // 设置组织代码
            sysTaskCycle.setOrganizeCode(AuthUtil.getOrganize().getOrganizeCode());

            sysTaskCycleService.insertSysTaskCycle(sysTaskCycle);
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (ParseException parseException) {
            return ResultBean.fail("表达式错误");
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 修改任务周期
     *
     * @param sysTaskCycle 任务周期
     * @return 处理结果
     */
    @PutMapping(value = "/update")
    @ApiOperation(value = "修改任务周期", notes = "修改任务周期")
    public ResultBean updateSysTaskCycle(SysTaskCycle sysTaskCycle) {
        try {
            // 数据校验
            ValidatorBean.beginValid(sysTaskCycle)
                    .notNull("id", sysTaskCycle.getId())
                    .notNull("name", sysTaskCycle.getName())
                    .notNull("taskCycleExps", sysTaskCycle.getTaskCycleExps());

            CronTriggerImpl trigger = new CronTriggerImpl();
            trigger.setCronExpression(sysTaskCycle.getTaskCycleExps());
            trigger.computeFirstFireTime(null);

            sysTaskCycleService.updateSysTaskCycle(sysTaskCycle);

            // 查询关联任务计划信息，并进行修改
            List<SysRefTaskCyclePlan> taskCyclePlans = sysTaskCycleService.findRefTaskCyclePlanByCycleId(sysTaskCycle.getId());
            SysTaskPlan sysTaskPlan;
            SysTask sysTask;
            for (SysRefTaskCyclePlan item : taskCyclePlans) {
                // 查找关联任务计划
                sysTaskPlan = sysTaskPlanService.getSysTaskPlanByNameAndGroup(item.getTaskPlanName(), item.getTaskPlanGroupName());
                // 查找任务
                sysTask = sysTaskService.getSysTaskById(sysTaskPlan.getTaskId());
                // 查询调度库中数据是否存在
                BaseResultBean baseResultBean =
                        getJobService(sysTaskPlan.getTaskSoftTypeRdd()).checkJobExists(sysTask.getTaskPackage(),
                                sysTask.getTaskClass());
                if (sysTaskPlan == null || sysTask == null || (baseResultBean.isSuccess() && (boolean) baseResultBean.getResultObject())) {
                    continue;
                }

                sysTaskPlan.setTaskCycleNameRdd(sysTaskCycle.getName());
                sysTaskPlan.setTaskCycleExpsRdd(sysTaskCycle.getTaskCycleExps());
                sysTaskPlan.setTaskCycleStartDateTimeRdd(sysTaskCycle.getTaskCycleStartDatetime());
                sysTaskPlan.setTaskCycleEndDateTimeRdd(sysTaskCycle.getTaskCycleEndDatetime());

                // 更新关联模块的定时任务
                getJobService(sysTaskPlan.getTaskSoftTypeRdd()).editTask(sysTask.getTaskPackage(), sysTask.getTaskClass(), sysTaskPlan.getName()
                        , sysTaskPlan.getGroupName(), sysTaskCycle.getTaskCycleExps(), sysTaskPlan.getTaskPlanDescription());

                // 停止或启用对应模块中的定时任务
                if (CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue() == sysTaskPlan.getTaskPlanStatusVal()) {
                    getJobService(sysTaskPlan.getTaskSoftTypeRdd()).resumeJob(sysTaskPlan.getName(), sysTaskPlan.getGroupName());
                } else if (CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue() == sysTaskPlan.getTaskPlanStatusVal()) {
                    getJobService(sysTaskPlan.getTaskSoftTypeRdd()).pauseJob(sysTaskPlan.getName(), sysTaskPlan.getGroupName());
                }
            }
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (ParseException parseException) {
            return ResultBean.fail("表达式错误");
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 删除任务周期
     *
     * @param idStr 任务周期id
     * @return 处理结果
     */
    @DeleteMapping(value = "/delete/{id}")
    @ApiOperation(value = "删除任务周期", notes = "删除任务周期")
    public ResultBean deleteSysTaskCycle(@PathVariable("id") String idStr) {
        try {
            startMultiService();

            // 数据校验
            ValidatorBean.checkNotNull(idStr, "时间表达式id 不能为空");
            if (sysTaskCycleService.getSysTaskCycleById(Long.parseLong(idStr)) == null) {
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                        .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION_DATA_NOT_EXIT.getCode())
                        .setErrorDetail("任务周期不存在")
                        .setErrorSolution("请重新选择")
                        .build();
            }

            sysTaskCycleService.deleteSysTaskCycleById(Long.parseLong(idStr));
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 查询所有任务周期信息
     *
     * @return 处理结果
     */
    @GetMapping(value = "/list")
    @ApiOperation(value = "查询所有任务周期信息", notes = "查询所有任务周期信息")
    public ResultBean<SysTaskCycle> findSysTaskCycle() {
        try {
            List<SysTaskCycle> list = sysTaskCycleService.listSysTaskCycle();
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode()).setResultList(list);
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 查询根据ID查询任务周期
     *
     * @param id 任务周期id
     * @return 处理结果
     */
    @GetMapping(value = "/get/{id}")
    @ApiOperation(value = "查询任务周期", notes = "查询根据ID查询任务周期")
    public ResultBean<SysTaskCycle> getSysTaskCycleById(@PathVariable("id") String id) {
        try {
            // 数据校验
            ValidatorBean.checkNotNull(id, "定时任务id 不能为空");

            SysTaskCycle sysTaskCycle = sysTaskCycleService.getSysTaskCycleById(Long.parseLong(id));
            if (sysTaskCycle != null) {
                return ResultBean.success("操作成功").setResultObject(sysTaskCycle).setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
            } else {
                return ResultBean.fail("数据不存在").setCode(ResourceEnumUtil.MESSAGE.EMPTY.getCode());
            }
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 任务周期查询，分页，排序
     *
     * @param sysTaskCycle 查询条件
     * @param pager        分页信息
     * @return 处理结果
     */
    @GetMapping("/query")
    @ApiOperation(value = "任务周期查询，分页，排序", notes = "任务周期查询，分页，排序")
    public ResultBean querySysTaskCycleByPager(SysTaskCycle sysTaskCycle, Pager pager) {
        try {
            ListPager listPager = sysTaskCycleService.querySysTaskCycleByPager(sysTaskCycle, pager);
            return ResultBean.success("操作成功").setListPager(listPager).setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 根据id批量删除任务周期
     *
     * @param idsStr 任务周期id
     * @return 处理结果
     */
    @DeleteMapping("/batch-delete")
    @ApiOperation(value = "根据id批量删除任务周期", notes = "根据id批量删除任务周期")
    public ResultBean deleteSysTaskTimeExpressionByIds(String[] idsStr) {
        try {
            if (idsStr.length == 0) {
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                        .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                        .setErrorDetail("请选择需要删除的资源。")
                        .build();
            }

            sysTaskCycleService.deleteSysTaskCycleByIds(StringTool.getArrayLong(idsStr));
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**************************************** 数据导入导出 ***********************************************/

    @GetMapping(value = "/export")
    @ApiOperation(value = "导出数据")
    public void exportExcel(SysTaskCycle bean, HttpServletResponse response) {
        File excelFile = null;
        try {
            Map<String, String> colMap = ExcelTool.getColName(SysTaskCycle.class);
            String[] colName = new String[colMap.size()];
            int j = 0;
            for (String key : colMap.keySet()) {
                colName[j++] = key;
            }

            ExcelTool excelTool = new ExcelTool(entityManager, RedisCacheTool.getImppRedis());
            // 将excel导出至临时文件夹
            excelFile = new File(System.getProperty("java.io.tmpdir") + File.separator + AuthUtil.getSessionUser().getUserName() +
                    "SysTaskCycle.xls");
            excelFile.createNewFile();
            excelTool.exportData(excelFile, sysTaskCycleService.findSysTaskCycleByCondition(bean), SysTaskCycle.class, colName);


            response.setContentType("application/vnd.ms-excel;charset=UTF-8");
            response.setCharacterEncoding("UTF-8");
            response.setHeader("Content-Disposition",
                    "attachment;fileName=" + URLEncoder.encode(excelFile.getName(), "UTF-8"));
            response.flushBuffer();

            BufferedInputStream bis = null;
            try {
                bis = new BufferedInputStream(new FileInputStream(excelFile));
                OutputStream os = response.getOutputStream();
                byte[] buffer = new byte[1024];
                int i = bis.read(buffer);
                while (i != -1) {
                    os.write(buffer, 0, i);
                    i = bis.read(buffer);
                }
            } catch (Exception e) {
                e.printStackTrace();
            } finally {
                if (bis != null) {
                    try {
                        bis.close();
                        excelFile.delete();
                    } catch (IOException e) {
                        e.printStackTrace();
                    }
                }
            }
        } catch (Exception e) {
            LOGGER.error("下载异常",e);
            throw ImppExceptionBuilder.newInstance().setErrorDetail(e.getMessage()).build();
        } finally {
            if (excelFile != null) {
                FileTool.deleteFile(excelFile.getPath());
            }
        }
    }

    @PostMapping(value = "/import")
    @ApiOperation(value = "导入数据")
    public ResultBean importExcel(@RequestParam("file") MultipartFile file) {
        try {
            ExcelTool excelTool = new ExcelTool(entityManager, RedisCacheTool.getImppRedis());
            List<SysTaskCycle> beanList = excelTool.importData(file.getOriginalFilename(), file.getInputStream(), SysTaskCycle.class);

            //校验导入数据并初始化
            for (SysTaskCycle taskCycle : beanList) {
                ConvertBean.modelInitialize(taskCycle, AuthUtil.getSessionUser());
            }

            sysTaskCycleService.insertBatchSysTaskCycle(beanList);

            return ResultBean.success("导入成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @GetMapping(value = "/down-template")
    @ApiOperation(value = "下载导入模板")
    public void downExcelTemplate(HttpServletResponse response) {
        try {
            ExcelTool excelTool = new ExcelTool(entityManager, RedisCacheTool.getImppRedis());
            String importTemplateCode = "SysTaskCycleImportTemplate";

            response.setContentType("application/vnd.ms-excel;charset=UTF-8");
            response.setCharacterEncoding("UTF-8");
            response.setHeader("Content-Disposition",
                    "attachment;fileName=" + URLEncoder.encode(importTemplateCode + ".xls", "UTF-8"));

            response.getOutputStream().write(excelTool.importTemplate(SysTaskCycle.class));
            response.getOutputStream().flush();
        } catch (Exception e) {
            LOGGER.error("下载异常",e);
           throw ImppExceptionBuilder.newInstance().setErrorDetail(e.getMessage()).build();
        }
    }
}
