package cn.estsh.i3plus.core.apiservice.mq;

import cn.estsh.i3plus.core.api.iservice.busi.ISysMenuService;
import cn.estsh.i3plus.platform.common.tool.JsonUtilTool;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.platform.bean.SysMenu;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.util.LocaleUtils;
import com.rabbitmq.client.Channel;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnExpression;
import org.springframework.context.annotation.Configuration;

import java.io.IOException;
import java.util.Map;

import static cn.estsh.i3plus.platform.common.util.PlatformConstWords.QUEUE_IMPP_RESOURCE;


/**
 * @Description :
 * @Reference :
 * @Author : wei.peng
 * @CreateDate : 19-8-23 下午3:12
 * @Modify:
 **/
@ConditionalOnExpression("'${impp.mq.queue.resource:true}' == 'true'")
@Configuration
public class SysLocalResourceQueue {

    private static final Logger LOGGER = LoggerFactory.getLogger(SysLocalResourceQueue.class);

    @Autowired
    private ISysMenuService sysMenuService;

    /**
     * 语言反向国际化实现
     * <per>
     * <br/>   根据信息创建不同语言的国际化信息。
     * <br/>        当存在的时候不做任何操作，不存的时候创建全部语言的资源信息
     * </per>
     * @param channel
     * @param message
     */
    @RabbitListener(queues = QUEUE_IMPP_RESOURCE)
    public void processImppMessage(String missResourceStr, Channel channel, Message message) {
        try {
            Map<String, String> missResource = JsonUtilTool.decode(missResourceStr, Map.class);
            if (missResource != null && !missResource.isEmpty()) {
                Map.Entry<String, String> entry = missResource.entrySet().iterator().next();
                SysMenu sysMenu = sysMenuService.getSysMenuByCode(entry.getValue());
                CommonEnumUtil.SOFT_TYPE softType = CommonEnumUtil.SOFT_TYPE.valueOf(sysMenu.getSoftType());
                if(softType == null){
                    softType = CommonEnumUtil.SOFT_TYPE.IMPP;
                }

                LocaleUtils.saveMissResource(entry.getKey(), AuthUtil.getSessionLanguage().toString(),
                        softType.getValue(),
                        CommonEnumUtil.SYS_LOCALE_RESOURCE_TYPE.MISSING.getValue(),
                        CommonEnumUtil.MiSS_RESOURCE_SOURCE.WEB.getValue());
            }
        }catch (Exception e){
            LOGGER.error("缺失资源保存异常：{}", missResourceStr, e);
        }finally {
            try {
                channel.basicAck(message.getMessageProperties().getDeliveryTag(),false);
                LOGGER.info("[MQ ACK] Tag :{} Completed successfully",message.getMessageProperties().getDeliveryTag());
            } catch (IOException e) {
                LOGGER.error("[MQ ERROR] MQ Ack Error Message:{}",e.getMessage());
                e.printStackTrace();
            }
        }
    }

}
