package cn.estsh.i3plus.core.apiservice.serviceimpl.base;

import cn.estsh.i3plus.core.api.iservice.base.ISysLogService;
import cn.estsh.i3plus.core.apiservice.util.NoSqlPack;
import cn.estsh.i3plus.platform.common.tool.TimeTool;
import cn.estsh.i3plus.platform.common.util.CommonConstWords;
import cn.estsh.i3plus.platform.plugin.elasticsearch.ElasticSearchPack;
import cn.estsh.i3plus.platform.plugin.elasticsearch.ElasticSearchTool;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.enumutil.ImppEnumUtil;
import cn.estsh.i3plus.pojo.platform.platbean.*;
import cn.estsh.impp.framework.boot.log.ImppLog;
import org.elasticsearch.search.aggregations.Aggregation;
import org.elasticsearch.search.aggregations.bucket.terms.ParsedTerms;
import org.elasticsearch.search.aggregations.bucket.terms.Terms;
import org.elasticsearch.search.aggregations.metrics.ParsedAvg;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * @Description :
 * @Reference :
 * @Author : wei.peng
 * @CreateDate : 20-2-13 下午4:16
 * @Modify:
 **/
@Service
public class SysLogElasticSearchService implements ISysLogService {

    public static final Logger LOGGER = LoggerFactory.getLogger(SysLogElasticSearchService.class);

    @Autowired
    private ImppLog imppLog;

    @Override
    public ListPager<SysLogException> queryLogExceptionPager(SysLogException log, Pager pager) {
        return ElasticSearchTool.searchPager(NoSqlPack.packEsSysLogException(log),pager,SysLogException.class);
    }

    @Override
    public List<SysLogException> findLogExceptionList(SysLogException log, Pager pager) {
        return ElasticSearchTool.searchList(NoSqlPack.packEsSysLogException(log),pager,SysLogException.class);
    }

    @Override
    public List<SysLogException> findLogExceptionList(SysLogException log) {
        return ElasticSearchTool.searchList(NoSqlPack.packEsSysLogException(log),SysLogException.class);
    }

    @Override
    public SysLogException insertLogException(SysLogException log) {
        imppLog.insertSysLogException(log);
        return log;
    }

    @Override
    public SysLogException getLogException(SysLogException log) {
        return ElasticSearchTool.getPrimaryKey(log.getId(),SysLogException.class);
    }

    @Override
    public void deleteLogException(SysLogException log) {
        ElasticSearchTool.deleteQuery(NoSqlPack.packEsSysLogException(log));
    }

    @Override
    public ListPager<SysLogOperate> queryLogOperatePager(SysLogOperate log, Pager pager) {
        return ElasticSearchTool.searchPager(NoSqlPack.packEsSysLogOperate(log),pager,SysLogOperate.class);
    }

    @Override
    public List<SysLogOperate> findLogOperateList(SysLogOperate log, Pager pager) {
        return ElasticSearchTool.searchList(NoSqlPack.packEsSysLogOperate(log),pager,SysLogOperate.class);
    }

    @Override
    public List<SysLogOperate> findLogOperateList(SysLogOperate log) {
        return ElasticSearchTool.searchList(NoSqlPack.packEsSysLogOperate(log),SysLogOperate.class);
    }

    @Override
    public SysLogOperate insertLogOperate(SysLogOperate log) {
        imppLog.insertSysLogOperate(log);
        return log;
    }

    @Override
    public SysLogOperate getLogOperate(SysLogOperate log) {
        return ElasticSearchTool.getPrimaryKey(log.getId(),SysLogOperate.class);
    }

    @Override
    public void deleteLogOperate(SysLogOperate log) {
        ElasticSearchTool.deleteQuery(NoSqlPack.packEsSysLogOperate(log));
    }

    @Override
    public ListPager<SysLogSystem> queryLogSystemPager(SysLogSystem log, Pager pager) {
        return ElasticSearchTool.searchPager(NoSqlPack.packEsSysLogSystem(log),pager,SysLogSystem.class);
    }

    @Override
    public List<SysLogSystem> findLogSystemList(SysLogSystem log, Pager pager) {
        return ElasticSearchTool.searchList(NoSqlPack.packEsSysLogSystem(log),pager,SysLogSystem.class);
    }

    @Override
    public List<SysLogSystem> findLogSystemList(SysLogSystem log) {
        return ElasticSearchTool.searchList(NoSqlPack.packEsSysLogSystem(log), SysLogSystem.class);
    }

    @Override
    public SysLogSystem insertLogSystem(SysLogSystem log) {
        imppLog.insertSysLogSystem(log);
        return log;
    }

    @Override
    public SysLogSystem getLogSystem(SysLogSystem log) {
        return ElasticSearchTool.getPrimaryKey(log.getId(),SysLogSystem.class);
    }

    @Override
    public void deleteLogSystem(SysLogSystem log) {
        ElasticSearchTool.deleteQuery(NoSqlPack.packEsSysLogSystem(log));
    }

    @Override
    public ListPager<SysLogTaskTime> queryLogTaskTimePager(SysLogTaskTime log, Pager pager) {
        return ElasticSearchTool.searchPager(NoSqlPack.packEsSysLogTaskTime(log),pager,SysLogTaskTime.class);
    }

    @Override
    public List<SysLogTaskTime> findLogTaskTimeList(SysLogTaskTime log, Pager pager) {
        return ElasticSearchTool.searchList(NoSqlPack.packEsSysLogTaskTime(log),pager,SysLogTaskTime.class);
    }

    @Override
    public List<SysLogTaskTime> findLogTaskTimeList(SysLogTaskTime log) {
        return ElasticSearchTool.searchList(NoSqlPack.packEsSysLogTaskTime(log),SysLogTaskTime.class);
    }

    @Override
    public SysLogTaskTime insertLogTaskTime(SysLogTaskTime log) {
        imppLog.insertSysLogTaskTime(log);
        return log;
    }

    @Override
    public SysLogTaskTime getLogTaskTime(SysLogTaskTime log) {
        return ElasticSearchTool.getPrimaryKey(log.getId(),SysLogTaskTime.class);
    }

    @Override
    public void deleteLogTaskTime(SysLogTaskTime log) {
         ElasticSearchTool.deleteQuery(NoSqlPack.packEsSysLogTaskTime(log));
    }
    /******************************************************* 定时任务日志 End *********************************************************/


    /******************************************************* 控制台日志 Start *********************************************************/

    @Override
    public ListPager<SysLogConsole> queryLogConsolePager(SysLogConsole log, Pager pager) {
        return ElasticSearchTool.searchPager(NoSqlPack.packEsSysLogConsole(log),pager,SysLogConsole.class);
    }

    @Override
    public List<SysLogConsole> findLogConsoleList(SysLogConsole log, Pager pager) {
        return ElasticSearchTool.searchList(NoSqlPack.packEsSysLogConsole(log),pager,SysLogConsole.class);
    }

    @Override
    public List<SysLogConsole> findLogConsoleList(SysLogConsole log) {
        return ElasticSearchTool.searchList(NoSqlPack.packEsSysLogConsole(log),SysLogConsole.class);
    }

    @Override
    public SysLogConsole insertLogConsoleTime(SysLogConsole log) {
        ElasticSearchPack pack = ElasticSearchPack.imppBuild(TimeTool.parseDate(log.getCreateDatetime()), SysLogConsole.class);
        ElasticSearchTool.insert(pack.getIndexName(),log);
        return log;
    }

    @Override
    public SysLogConsole getLogConsoleTime(SysLogConsole log) {
        return ElasticSearchTool.getPrimaryKey(log.getId(),SysLogConsole.class);
    }

    @Override
    public void deleteLogConsoleTime(SysLogConsole log) {
        ElasticSearchTool.deleteQuery(NoSqlPack.packEsSysLogConsole(log));
    }


    /******************************************************* 控制台日志 End *********************************************************/

    @Override
    public void actionCleanLog(String startTime, String endTime) {
        ElasticSearchTool.reload();
        try{
            SysLogOperate operate = new SysLogOperate();
            operate.setCreateDatetime(startTime);
            operate.setCreateDateTimeStart(startTime);
            operate.setCreateDateTimeEnd(endTime);
            deleteLogOperate(operate);
        }catch (Exception e){
        }

        try {
            SysLogSystem system = new SysLogSystem();
            system.setCreateDatetime(startTime);
            system.setCreateDateTimeStart(startTime);
            system.setCreateDateTimeEnd(endTime);
            deleteLogSystem(system);
        }catch (Exception e){
        }

        try {
            SysLogException exception = new SysLogException();
            exception.setCreateDatetime(startTime);
            exception.setCreateDateTimeStart(startTime);
            exception.setCreateDateTimeEnd(endTime);
            deleteLogException(exception);
        }catch (Exception e){
        }

        try {
            SysLogTaskTime taskTime = new SysLogTaskTime();
            taskTime.setCreateDatetime(startTime);
            taskTime.setCreateDateTimeStart(startTime);
            taskTime.setCreateDateTimeEnd(endTime);
            deleteLogTaskTime(taskTime);
        }catch (Exception e){
        }

        try {
            SysLogConsole console = new SysLogConsole();
            console.setCreateDatetime(startTime);
            console.setCreateDateTimeStart(startTime);
            console.setCreateDateTimeEnd(endTime);
            deleteLogConsoleTime(console);
        }catch (Exception e){
        }
    }

    @Override
    public Map<String, Object> querySysLogSystemAvgExecuteTime(String startTime, String endTime) {
        Map resultData = new LinkedHashMap();
        ElasticSearchPack pack = ElasticSearchPack.imppBuild(SysLogSystem.class);
        pack.pager(CommonConstWords.getDefaultPager());
        pack.andStringDateTimeBetween("createDatetime",startTime,endTime);
        pack.group("createDatetime").avg("executeTime").buildAggregation();
        Map<String, Aggregation> aggregationMap = ElasticSearchTool.searchAggregationMap(pack);

        ParsedTerms parsedTerms = (ParsedTerms)aggregationMap.get("group_createDatetime");
        if(Objects.nonNull(parsedTerms)){
            for (Terms.Bucket bucket : parsedTerms.getBuckets()) {
                try {
                    String key = bucket.getKeyAsString();
                    Double value = ((ParsedAvg) bucket.getAggregations().get("avg_executeTime")).getValue();
                    resultData.put(key,value.intValue());
                }catch (Exception e){
                }
            }
        }

        return resultData;
    }

    @Override
    public Map<String, Object> querySysLogSystemByLevel(String startTime, String endTime) {
        Map resultData = new LinkedHashMap();
        ElasticSearchPack pack = ElasticSearchPack.imppBuild(SysLogSystem.class);
        pack.pager(CommonConstWords.getDefaultPager());
        pack.andStringDateTimeBetween("createDatetime",startTime,endTime);
        pack.group("logLevel").count("id").buildAggregation();
        Map<String, Aggregation> aggregationMap = ElasticSearchTool.searchAggregationMap(pack);


        ParsedTerms parsedTerms = (ParsedTerms)aggregationMap.get("group_logLevel");
        if(Objects.nonNull(parsedTerms)){
            for (Terms.Bucket bucket : parsedTerms.getBuckets()) {
                try {
                    String key = bucket.getKeyAsString();
                    resultData.put(ImppEnumUtil.LOG_LEVEL.valueOfName(Integer.parseInt(key)),bucket.getDocCount());
                }catch (Exception e){
                }
            }
        }

        return resultData;
    }

    /******************************************************* 系统异常日志 End *********************************************************/


}
