package cn.estsh.i3plus.core.apiservice.serviceimpl.base;

import cn.estsh.i3plus.core.api.iservice.base.ISysLogService;
import cn.estsh.i3plus.core.apiservice.util.NoSqlPack;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.platform.common.tool.MathOperation;
import cn.estsh.i3plus.platform.common.tool.StringTool;
import cn.estsh.i3plus.platform.common.util.CommonConstWords;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.common.PagerHelper;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.enumutil.ImppEnumUtil;
import cn.estsh.i3plus.pojo.platform.platbean.*;
import cn.estsh.i3plus.pojo.platform.platrepositorymongo.SysLogExceptionRepository;
import cn.estsh.i3plus.pojo.platform.platrepositorymongo.SysLogOperateRepository;
import cn.estsh.i3plus.pojo.platform.platrepositorymongo.SysLogSystemRepository;
import cn.estsh.i3plus.pojo.platform.platrepositorymongo.SysLogTaskTimeRepository;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ImppSwitchUtil;
import cn.estsh.impp.framework.boot.util.SpringContextsUtil;
import com.mongodb.client.AggregateIterable;
import com.mongodb.client.MongoCursor;
import org.bson.Document;
import org.bson.conversions.Bson;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.data.mongodb.MongoDbFactory;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

/**
 * @Description :
 * @Reference :
 * @Author : wei.peng
 * @CreateDate : 20-2-13 下午4:16
 * @Modify:
 **/
@Service
public class SysLogMongoService implements ISysLogService {

    public static final Logger LOGGER = LoggerFactory.getLogger(SysLogMongoService.class);

    private SysLogTaskTimeRepository logTaskTimeRDao;
    private SysLogSystemRepository logSystemRDao;
    private SysLogOperateRepository logOperateRDao;
    private SysLogExceptionRepository logExceptionRDao;
    private MongoDbFactory mongoDbFactory;


    public SysLogTaskTimeRepository getSysLogTaskTimeRepository() {
        if (logTaskTimeRDao == null && ImppSwitchUtil.isSwitchMongo()) {
            try {
                logTaskTimeRDao = SpringContextsUtil.getBean(SysLogTaskTimeRepository.class);
            }catch (Exception e){
            }
        }

        if (logTaskTimeRDao == null) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.IMPP.getCode())
                    .setErrorCode(ImppExceptionEnum.LOG_SAVE_STORAGE_ERROR.getCode())
                    .setErrorDetail("日志Mongo存储错误")
                    .setErrorSolution("请检查是否开启MongoDB存储")
                    .build();
        }

        return logTaskTimeRDao;
    }

    public SysLogSystemRepository getSysLogSystemRepository() {
        if (logSystemRDao == null && ImppSwitchUtil.isSwitchMongo()) {
            try {
                logSystemRDao = SpringContextsUtil.getBean(SysLogSystemRepository.class);
            }catch (Exception e){
            }
        }

        if (logSystemRDao == null) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.IMPP.getCode())
                    .setErrorCode(ImppExceptionEnum.LOG_SAVE_STORAGE_ERROR.getCode())
                    .setErrorDetail("日志Mongo存储错误")
                    .setErrorSolution("请检查是否开启MongoDB存储")
                    .build();
        }

        return logSystemRDao;
    }

    public SysLogOperateRepository getSysLogOperateRepository() {
        if (logOperateRDao == null && ImppSwitchUtil.isSwitchMongo()) {
            try {
                logOperateRDao = SpringContextsUtil.getBean(SysLogOperateRepository.class);
            }catch (Exception e){
            }
        }

        if (logOperateRDao == null) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.IMPP.getCode())
                    .setErrorCode(ImppExceptionEnum.LOG_SAVE_STORAGE_ERROR.getCode())
                    .setErrorDetail("日志Mongo存储错误")
                    .setErrorSolution("请检查是否开启MongoDB存储")
                    .build();
        }

        return logOperateRDao;
    }

    public SysLogExceptionRepository getSysLogExceptionRepository() {
        if (logExceptionRDao == null && ImppSwitchUtil.isSwitchMongo()) {
            try {
                logExceptionRDao = SpringContextsUtil.getBean(SysLogExceptionRepository.class);
            }catch (Exception e){
            }
        }

        if (logExceptionRDao == null) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.IMPP.getCode())
                    .setErrorCode(ImppExceptionEnum.LOG_SAVE_STORAGE_ERROR.getCode())
                    .setErrorDetail("日志Mongo存储错误")
                    .setErrorSolution("请检查是否开启MongoDB存储")
                    .build();
        }

        return logExceptionRDao;
    }

    public MongoDbFactory getMongoDbFactory() {
        if (mongoDbFactory == null && ImppSwitchUtil.isSwitchMongo()) {
            try {
                mongoDbFactory = SpringContextsUtil.getBean(MongoDbFactory.class);
            }catch (Exception e){
            }
        }

        if (mongoDbFactory == null) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.IMPP.getCode())
                    .setErrorCode(ImppExceptionEnum.LOG_SAVE_STORAGE_ERROR.getCode())
                    .setErrorDetail("日志Mongo存储错误")
                    .setErrorSolution("请检查是否开启MongoDB存储")
                    .build();
        }

        return mongoDbFactory;
    }



    @Override
    public ListPager<SysLogException> queryLogExceptionPager(SysLogException log, Pager pager) {
        Bson bson = NoSqlPack.packMongoBySysLogException(log);
        pager = PagerHelper.getPager(pager, getSysLogTaskTimeRepository().findByBsonCount(bson));
        return new ListPager(getSysLogExceptionRepository().findByBsonPager(bson, pager, log.getOrderByParam(), log.getAscOrDesc()), pager);
    }

    @Override
    public List<SysLogException> findLogExceptionList(SysLogException log, Pager pager) {
        Bson bson = NoSqlPack.packMongoBySysLogException(log);
        return getSysLogExceptionRepository().findByBsonPager(bson, pager, log.getOrderByParam(), log.getAscOrDesc());
    }

    @Override
    public List<SysLogException> findLogExceptionList(SysLogException log) {
        Bson bson = NoSqlPack.packMongoBySysLogException(log);
        return getSysLogExceptionRepository().findByBsonPager(bson, CommonConstWords.getDefaultPager(), log.getOrderByParam(), log.getAscOrDesc());
    }

    @Override
    public SysLogException insertLogException(SysLogException log) {
        return getSysLogExceptionRepository().save(log);
    }

    @Override
    public SysLogException getLogException(SysLogException log) {
        return getSysLogExceptionRepository().getById(log.getId());
    }

    @Override
    public void deleteLogException(SysLogException log) {
        getSysLogExceptionRepository().deleteByProperty(NoSqlPack.packMongoBySysLogException(log));
    }

    @Override
    public ListPager<SysLogOperate> queryLogOperatePager(SysLogOperate log, Pager pager) {
        Bson bson = NoSqlPack.packMongoByLogOperate(log);
        pager = PagerHelper.getPager(pager, getSysLogTaskTimeRepository().findByBsonCount(bson));
        return new ListPager(getSysLogOperateRepository().findByBsonPager(bson, pager, log.getOrderByParam(), log.getAscOrDesc()), pager);
    }

    @Override
    public List<SysLogOperate> findLogOperateList(SysLogOperate log, Pager pager) {
        Bson bson = NoSqlPack.packMongoByLogOperate(log);
        return getSysLogOperateRepository().findByBsonPager(bson, pager, log.getOrderByParam(), log.getAscOrDesc());
    }

    @Override
    public List<SysLogOperate> findLogOperateList(SysLogOperate log) {
        Bson bson = NoSqlPack.packMongoByLogOperate(log);
        return getSysLogOperateRepository().findByBsonPager(bson, CommonConstWords.getDefaultPager(), log.getOrderByParam(), log.getAscOrDesc());
    }

    @Override
    public SysLogOperate insertLogOperate(SysLogOperate log) {
        return getSysLogOperateRepository().save(log);
    }

    @Override
    public SysLogOperate getLogOperate(SysLogOperate log) {
        return getSysLogOperateRepository().getById(log.getId());
    }

    @Override
    public void deleteLogOperate(SysLogOperate log) {
        getSysLogOperateRepository().deleteByProperty(NoSqlPack.packMongoByLogOperate(log));
    }

    @Override
    public ListPager<SysLogSystem> queryLogSystemPager(SysLogSystem log, Pager pager) {
        Bson bson = NoSqlPack.packMongoByLogSystem(log);
        pager = PagerHelper.getPager(pager, getSysLogTaskTimeRepository().findByBsonCount(bson));
        return new ListPager(getSysLogSystemRepository().findByBsonPager(bson, pager, log.getOrderByParam(), log.getAscOrDesc()), pager);
    }

    @Override
    public List<SysLogSystem> findLogSystemList(SysLogSystem log, Pager pager) {
        Bson bson = NoSqlPack.packMongoByLogSystem(log);
        return getSysLogSystemRepository().findByBsonPager(bson, pager, log.getOrderByParam(), log.getAscOrDesc());
    }

    @Override
    public List<SysLogSystem> findLogSystemList(SysLogSystem log) {
        Bson bson = NoSqlPack.packMongoByLogSystem(log);
        return getSysLogSystemRepository().findByBsonPager(bson, CommonConstWords.getDefaultPager(), log.getOrderByParam(), log.getAscOrDesc());
    }

    @Override
    public SysLogSystem insertLogSystem(SysLogSystem log) {
        return getSysLogSystemRepository().save(log);
    }

    @Override
    public SysLogSystem getLogSystem(SysLogSystem log) {
        return getSysLogSystemRepository().getById(log.getId());
    }

    @Override
    public void deleteLogSystem(SysLogSystem log) {
        getSysLogSystemRepository().deleteByProperty(NoSqlPack.packMongoByLogSystem(log));
    }

    @Override
    public ListPager<SysLogTaskTime> queryLogTaskTimePager(SysLogTaskTime log, Pager pager) {
        Bson bson = NoSqlPack.packMongoByLogTaskTime(log);
        pager = PagerHelper.getPager(pager, getSysLogTaskTimeRepository().findByBsonCount(bson));
        return new ListPager(getSysLogTaskTimeRepository().findByBsonPager(bson, pager, log.getOrderByParam(), log.getAscOrDesc()), pager);
    }

    @Override
    public List<SysLogTaskTime> findLogTaskTimeList(SysLogTaskTime log, Pager pager) {
        Bson bson = NoSqlPack.packMongoByLogTaskTime(log);
        return getSysLogTaskTimeRepository().findByBsonPager(bson, pager, log.getOrderByParam(), log.getAscOrDesc());
    }

    @Override
    public List<SysLogTaskTime> findLogTaskTimeList(SysLogTaskTime log) {
        Bson bson = NoSqlPack.packMongoByLogTaskTime(log);
        return getSysLogTaskTimeRepository().findByBsonPager(bson, CommonConstWords.getDefaultPager(), log.getOrderByParam(), log.getAscOrDesc());
    }

    @Override
    public SysLogTaskTime insertLogTaskTime(SysLogTaskTime log) {
        return getSysLogTaskTimeRepository().save(log);
    }

    @Override
    public SysLogTaskTime getLogTaskTime(SysLogTaskTime log) {
        return getSysLogTaskTimeRepository().getById(log.getId());
    }

    @Override
    public void deleteLogTaskTime(SysLogTaskTime log) {
        getSysLogTaskTimeRepository().deleteByProperty(NoSqlPack.packMongoByLogTaskTime(log));
    }

    @Override
    public ListPager<SysLogConsole> queryLogConsolePager(SysLogConsole log, Pager pager) {
        throw ImppExceptionBuilder.newInstance()
                .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                .setErrorCode(cn.estsh.impp.framework.boot.exception.ImppExceptionEnum.VARIFY_EXCEPTION_DATA_EXIT.getCode())
                .setErrorDetail("日志数据量太大,推荐使用Elasticsearch技术存储")
                .setErrorSolution("请重新操作")
                .build();
    }

    @Override
    public List<SysLogConsole> findLogConsoleList(SysLogConsole log, Pager pager) {
        throw ImppExceptionBuilder.newInstance()
                .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                .setErrorCode(cn.estsh.impp.framework.boot.exception.ImppExceptionEnum.DATABASE_EXCEPTION.getCode())
                .setErrorDetail("日志数据量太大,推荐使用Elasticsearch技术存储")
                .setErrorSolution("请重新操作")
                .build();
    }

    @Override
    public List<SysLogConsole> findLogConsoleList(SysLogConsole log) {
        throw ImppExceptionBuilder.newInstance()
                .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                .setErrorCode(cn.estsh.impp.framework.boot.exception.ImppExceptionEnum.DATABASE_EXCEPTION.getCode())
                .setErrorDetail("日志数据量太大,推荐使用Elasticsearch技术存储")
                .setErrorSolution("请重新操作")
                .build();
    }

    @Override
    public SysLogConsole insertLogConsoleTime(SysLogConsole log) {
        throw ImppExceptionBuilder.newInstance()
                .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                .setErrorCode(cn.estsh.impp.framework.boot.exception.ImppExceptionEnum.DATABASE_EXCEPTION.getCode())
                .setErrorDetail("日志数据量太大,推荐使用Elasticsearch技术存储")
                .setErrorSolution("请重新操作")
                .build();
    }

    @Override
    public SysLogConsole getLogConsoleTime(SysLogConsole log) {
        throw ImppExceptionBuilder.newInstance()
                .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                .setErrorCode(cn.estsh.impp.framework.boot.exception.ImppExceptionEnum.DATABASE_EXCEPTION.getCode())
                .setErrorDetail("日志数据量太大,推荐使用Elasticsearch技术存储")
                .setErrorSolution("请重新操作")
                .build();
    }

    @Override
    public void deleteLogConsoleTime(SysLogConsole log) {
        throw ImppExceptionBuilder.newInstance()
                .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                .setErrorCode(cn.estsh.impp.framework.boot.exception.ImppExceptionEnum.DATABASE_EXCEPTION.getCode())
                .setErrorDetail("日志数据量太大,推荐使用Elasticsearch技术存储")
                .setErrorSolution("请重新操作")
                .build();
    }

    /******************************************************* 定时任务日志 End *********************************************************/


    @Override
    public void actionCleanLog(String startTime, String endTime) {
        try{
            SysLogOperate operate = new SysLogOperate();
            operate.setCreateDatetime(startTime);
            operate.setCreateDateTimeStart(startTime);
            operate.setCreateDateTimeEnd(endTime);
            deleteLogOperate(operate);
        }catch (Exception e){
        }

        try {
            SysLogSystem system = new SysLogSystem();
            system.setCreateDatetime(startTime);
            system.setCreateDateTimeStart(startTime);
            system.setCreateDateTimeEnd(endTime);
            deleteLogSystem(system);
        }catch (Exception e){
        }

        try {
            SysLogException exception = new SysLogException();
            exception.setCreateDatetime(startTime);
            exception.setCreateDateTimeStart(startTime);
            exception.setCreateDateTimeEnd(endTime);
            deleteLogException(exception);
        }catch (Exception e){
        }

        try {
            SysLogTaskTime taskTime = new SysLogTaskTime();
            taskTime.setCreateDatetime(startTime);
            taskTime.setCreateDateTimeStart(startTime);
            taskTime.setCreateDateTimeEnd(endTime);
            deleteLogTaskTime(taskTime);
        }catch (Exception e){
        }

        try {
            SysLogConsole console = new SysLogConsole();
            console.setCreateDatetime(startTime);
            console.setCreateDateTimeStart(startTime);
            console.setCreateDateTimeEnd(endTime);
            deleteLogConsoleTime(console);
        }catch (Exception e){
        }
    }

    @Override
    public Map<String, Object> querySysLogSystemAvgExecuteTime(String startTime, String endTime) {
        // 筛选
        Document sub_match = new Document();
        sub_match.put("createDatetime", new Document("$gte", startTime).append("$lte", endTime));
        // 分组
        Document sub_group = new Document();
        sub_group.put("_id", "$createDatetime");
        sub_group.put("avg", new Document("$avg", "$executeTime"));


        Document match = new Document("$match", sub_match);
        Document group = new Document("$group", sub_group);
        Document sort = new Document("$sort", new Document("_id", -1));

        List<Document> aggregateList = new ArrayList<>();
        aggregateList.add(match);
        aggregateList.add(group);
        aggregateList.add(sort);

        AggregateIterable<Document> findIter = getMongoDbFactory().getDb()
                .getCollection(StringTool.toLowerCaseFirstOne(SysLogSystem.class.getSimpleName()))
                .aggregate(aggregateList);
        MongoCursor<Document> cursor = findIter.iterator();

        Map resultData = new LinkedHashMap();
        Document item_doc;
        while (cursor.hasNext()) {
            item_doc = cursor.next();
            resultData.put(item_doc.getString("_id"), MathOperation.round(item_doc.getDouble("avg"), 0));
        }
        return resultData;
    }

    @Override
    public Map<String, Object> querySysLogSystemByLevel(String startTime, String endTime) {
        // 筛选
        Document sub_match = new Document();
        sub_match.put("createDatetime", new Document("$gte", startTime).append("$lte", endTime));
        // 分组
        Document sub_group = new Document();
        sub_group.put("_id", "$logLevel");
        sub_group.put("count", new Document("$sum", 1));

        Document match = new Document("$match", sub_match);
        Document group = new Document("$group", sub_group);

        List<Document> aggregateList = new ArrayList<>();
        aggregateList.add(match);
        aggregateList.add(group);

        AggregateIterable<Document> findIter = getMongoDbFactory().getDb()
                .getCollection(StringTool.toLowerCaseFirstOne(SysLogSystem.class.getSimpleName()))
                .aggregate(aggregateList);
        MongoCursor<Document> cursor = findIter.iterator();

        Map resultData = new LinkedHashMap();
        Document item_doc;
        while (cursor.hasNext()) {
            item_doc = cursor.next();
            resultData.put(ImppEnumUtil.LOG_LEVEL.valueOfName(item_doc.getInteger("_id")), MathOperation.round(item_doc.getInteger("count"), 0));
        }

        return resultData;
    }

    /******************************************************* 系统异常日志 End *********************************************************/


}
