package cn.estsh.i3plus.core.apiservice.serviceimpl.busi;

import cn.estsh.i3plus.core.api.iservice.busi.*;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.enumutil.ModelEnumUtil;
import cn.estsh.i3plus.pojo.model.platform.CommonTreeModel;
import cn.estsh.i3plus.pojo.platform.bean.*;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @Description :
 * @Reference :
 * @Author : wei.peng
 * @CreateDate : 2019-07-04 下午12:46
 * @Modify:
 **/
@Service
public class CoreMemTreeService implements ICoreMemTreeService {

    @Autowired
    private IPersonnelService personnelService;

    @Autowired
    private ISysMenuService menuService;

    @Autowired
    private ISysPositionService positionService;
    @Autowired
    private ISysOrganizeService organizeService;
    @Autowired
    private ISysDepartmentService departmentService;

    @Override
    public List<SysPosition> packTreeSysPosition(List<SysPosition> list, Long parentId) {
        List<SysPosition> result = new ArrayList<>();

        if (list != null && list.size() > 0) {
            List<SysPosition> copyList = new ArrayList<>(list);
            list.forEach(position -> {
                if (position.getParentId() != null && position.getParentId().equals(parentId)) {
                    result.add(position);
                    copyList.remove(position);
                    position.setChildList(packTreeSysPosition(copyList, position.getId()));
                }
            });
        }

        return result;
    }

    @Override
    public List<SysPosition> packTreeSysPositionBySysRefUserPosition(List<SysPosition> list, List<SysRefUserPosition> refList) {
        if (refList == null || refList.isEmpty()) {
            return Collections.emptyList();
        }
        List<Long> ids = new ArrayList<>(refList.size());
        refList.forEach(ref -> ids.add(ref.getPositionId()));

        return packTreeSysPositionByIds(list, ids);
    }

    @Override
    public List<SysPosition> packTreeSysPositionByIds(List<SysPosition> list, List<Long> ids) {
        List<SysPosition> result = new ArrayList<>();

        if (list != null && list.size() > 0) {
            list.forEach(position -> {
                if (position.getChildList() == null || position.getChildList().size() <= 0) {
                    if (ids.contains(position.getId())) {
                        result.add(position);
                    }
                } else {
                    List<SysPosition> childList = packTreeSysPositionByIds(position.getChildList(), ids);
                    if (ids.contains(position.getId()) || childList != null && childList.size() > 0) {
                        position.setChildList(childList);
                        result.add(position);
                    }
                }
            });
        }

        return result;
    }

    @Override
    public SysPosition packTreeSysPositionById(List<SysPosition> list, Long id) {
        if (list != null && list.size() > 0) {
            for (SysPosition position : list) {
                if (position.getId().equals(id)) {
                    position.setChildList(packTreeSysPosition(list, position.getId()));
                    return position;
                }
            }
        }
        return null;
    }

    @Override
    public List<Long> findChildSysPosition(SysPosition position) {
        List<Long> result = new ArrayList<>();

        if (position != null && position.getChildList() != null && position.getChildList().size() > 0) {
            for (SysPosition sp : position.getChildList()) {
                result.add(sp.getId());
                List<Long> list = findChildSysPosition(sp);
                result.addAll(list);
            }
        }

        return result;
    }

    @Override
    public List<Long> findSysPositionChildIdList(Long id) {
        SysPosition position = positionService.get(id);
        if (position != null) {
            List<SysPosition> list = positionService.findAll();
            if (list != null && list.size() > 0) {
                SysPosition treeSPosition = packTreeSysPositionById(list, position.getId());
                List<Long> idList = findChildSysPosition(treeSPosition);
                idList.add(position.getId());
                return idList;
            }
        }
        return Collections.emptyList();
    }

    /********************************************* Start SysOrganize Tree *********************************************/
    @Override
    public List<SysOrganize> packTreeSysOrganize(List<SysOrganize> list, Long parentId) {
        List<SysOrganize> result = new ArrayList<>();

        if (list != null && list.size() > 0) {
            List<SysOrganize> copyList = new ArrayList<>(list);
            list.forEach(organize -> {
                if (organize.getParentId() != null && organize.getParentId().equals(parentId)) {
                    result.add(organize);
                    copyList.remove(organize);
                    organize.setChildList(packTreeSysOrganize(copyList, organize.getId()));
                }
            });
        }

        return result;
    }

    @Override
    public List<SysOrganize> packTreeSysOrganizeBySysRefUserOrganize(List<SysOrganize> list, List<SysRefUserDepartment> refList) {
        List<Long> ids = new ArrayList<>(refList.size());

        if (refList != null && refList.size() > 0) {
            refList.forEach(ref -> ids.add(ref.getOrganizeId()));
        }

        return packTreeSysOrganizeByIds(list, ids);
    }

    @Override
    public List<SysOrganize> packTreeSysOrganizeBySysRefUserInfoOrganize(List<SysOrganize> list, List<SysRefUserInfoDepartment> refList) {
        List<Long> ids = new ArrayList<>(refList.size());

        if (refList != null && refList.size() > 0) {
            refList.forEach(ref -> {
                ids.add(ref.getOrganizeId());
            });
        }

        return packTreeSysOrganizeByIds(list, ids);
    }

    @Override
    public List<SysOrganize> packTreeSysOrganizeByIds(List<SysOrganize> list, List<Long> ids) {
        List<SysOrganize> result = new ArrayList<>();

        if (list != null && list.size() > 0) {
            list.forEach(organize -> {
                if (organize.getChildList() == null || organize.getChildList().size() <= 0) {
                    if (ids.contains(organize.getId())) {
                        result.add(organize);
                    }
                } else {
                    List<SysOrganize> childList = packTreeSysOrganizeByIds(organize.getChildList(), ids);
                    if (ids.contains(organize.getId()) || childList != null && childList.size() > 0) {
                        organize.setChildList(childList);
                        result.add(organize);
                    }
                }
            });
        }

        return result;
    }

    @Override
    public SysOrganize packTreeSysOrganizeById(List<SysOrganize> list, Long id) {
        if (list != null && list.size() > 0) {
            for (SysOrganize organize : list) {
                if (organize.getId().equals(id)) {
                    organize.setChildList(packTreeSysOrganize(list, organize.getId()));
                    return organize;
                }
            }
        }
        return null;
    }

    @Override
    public SysOrganize getTreeSysOrganizeById(List<SysOrganize> list, Long id) {
        SysOrganize result = null;
        if (list != null && list.size() > 0) {
            for (SysOrganize organize : list) {
                if (organize.getId().equals(id)) {
                    result = organize;
                } else if (organize.getChildList() != null && organize.getChildList().size() > 0) {
                    result = getTreeSysOrganizeById(organize.getChildList(), id);
                }
                if (result != null) {
                    return result;
                }
            }
        }
        return result;
    }

    @Override
    public List<Long> findChildSysOrganize(SysOrganize organize) {
        List<Long> result = new ArrayList<>();

        if (organize != null) {
            if (organize.getChildList() != null && organize.getChildList().size() > 0) {
                for (SysOrganize so : organize.getChildList()) {
                    result.add(so.getId());
                    result.addAll(findChildSysOrganize(so));
                }
            }
            result.add(organize.getId());
        }

        return result;
    }

    @Override
    public List<Long> findSysOrganizeChildIdList(Long id) {
        SysOrganize organize = organizeService.get(id);
        if (organize != null) {
            List<SysOrganize> list = organizeService.findAll();
            if (list != null && list.size() > 0) {
                SysOrganize treeSPosition = packTreeSysOrganizeById(list, organize.getId());
                List<Long> idList = findChildSysOrganize(treeSPosition);
                idList.add(organize.getId());
                return idList;
            }
        }
        return Collections.emptyList();
    }

    /********************************************* End SysOrganize Tree *********************************************/


    /********************************************* Start CommonTreeModel Tree *********************************************/

    @Override
    public List<CommonTreeModel> packTreeTreeModelByRefUserDepartment(List<SysOrganize> organizeList, List<SysDepartment> departmentList, List<SysRefUserDepartment> refList) {
        if (refList != null && organizeList != null && departmentList != null) {
            List<Long> organizeIdList = new ArrayList<>();
            List<Long> departmentIdList = new ArrayList<>();

            refList.forEach(ref -> {
                if (!organizeIdList.contains(ref.getOrganizeId())) {
                    organizeIdList.add(ref.getOrganizeId());
                }
                if (!departmentIdList.contains(ref.getDepartmentId())) {
                    departmentIdList.add(ref.getDepartmentId());
                }
            });

            organizeList = packTreeSysOrganize(organizeList, CommonEnumUtil.PARENT.DEFAULT.getValue());
            organizeList = packTreeSysOrganizeByIds(organizeList, organizeIdList);

            departmentList = packTreeSysDepartment(departmentList, CommonEnumUtil.PARENT.DEFAULT.getValue());
            departmentList = packTreeSysDepartmentByIds(departmentList, departmentIdList);

            List<CommonTreeModel> modelList = packTreeTreeModelSysOrganize(organizeList);
            List<CommonTreeModel> result = packTreeTreeModel(modelList, departmentList);
            return result;
        }

        return Collections.emptyList();
    }

    @Override
    public List<CommonTreeModel> packTreeTreeModelRoot() {
        List<SysOrganize> organizeList = organizeService.findAll();
        List<SysDepartment> departmentList = departmentService.findAll();

        organizeList = packTreeSysOrganize(organizeList, CommonEnumUtil.PARENT.DEFAULT.getValue());
        departmentList = packTreeSysDepartment(departmentList, CommonEnumUtil.PARENT.DEFAULT.getValue());

        List<CommonTreeModel> modelList = packTreeTreeModelSysOrganize(organizeList);
        List<CommonTreeModel> result = packTreeTreeModel(modelList, departmentList);
        return result;
    }

    @Override
    public List<CommonTreeModel> packTreeTreeModelByUserId(Long userId) {
        List<CommonTreeModel> result = null;

        // 查出用户权限
        List<SysRefUserDepartment> refList = personnelService.findSysRefUserDepartmentByUserId(userId);
        if (refList != null && refList.size() > 0) {
            // 分装组织树
            List<SysOrganize> organizeList = organizeService.findAll();
            // 分装部门树
            List<SysDepartment> departmentList = departmentService.findAll();
            // 分装组合树
            result = packTreeTreeModelByRefUserDepartment(organizeList, departmentList, refList);
        }

        return result;
    }

    @Override
    public List<CommonTreeModel> packTreeTreeModelSysOrganize(List<SysOrganize> list) {
        List<CommonTreeModel> result = new ArrayList<>();

        if (list != null && list.size() > 0) {
            for (SysOrganize organize : list) {
                CommonTreeModel model = new CommonTreeModel();
                model.setBeanType(ModelEnumUtil.COMMON_TREE_TYPE.TYPE_ORGANIZE.getValue());
                model.setBean(organize);
                model.getChildList().addAll(packTreeTreeModelSysOrganize(organize.getChildList()));

                result.add(model);
            }
        }

        return result;
    }

    @Override
    public List<CommonTreeModel> packTreeTreeModelSysDepartment(List<SysDepartment> list) {
        List<CommonTreeModel> result = new ArrayList<>();

        if (list != null && list.size() > 0) {
            for (SysDepartment department : list) {
                CommonTreeModel model = new CommonTreeModel();
                model.setBeanType(ModelEnumUtil.COMMON_TREE_TYPE.TYPE_DEPARTMENT.getValue());
                model.setBean(department);
                model.getChildList().addAll(packTreeTreeModelSysDepartment(department.getChildList()));

                result.add(model);
            }
        }

        return result;
    }

    @Override
    public List<CommonTreeModel> packTreeTreeModel(List<CommonTreeModel> list, List<SysDepartment> departmentList) {
        if (list != null && list.size() > 0 && departmentList != null && departmentList.size() > 0) {
            list.forEach(model -> {
                SysOrganize organize = (SysOrganize) model.getBean();
                List<SysDepartment> departments = new ArrayList<>();
                departmentList.forEach(department -> {
                    if (department.getParentId().equals(CommonEnumUtil.PARENT.DEFAULT.getValue()) && department.getOrganizeId().equals(organize.getId())) {
                        departments.add(department);
                    }
                });
                List<CommonTreeModel> childList = new ArrayList<>();
                childList.addAll(packTreeTreeModel(model.getChildList(), departmentList));
                childList.addAll(packTreeTreeModelSysDepartment(departments));
                model.setChildList(childList);
            });
        }
        return list;
    }

    @Override
    public List<Long> findDepartmentIdListByOrganizeIdAndDepartmentIdAndUserId(Long organizeId, Long departmentId, Long userId) {
        List<Long> refFilterIds = new ArrayList<>();
        List<Long> refIdList = findDepartmentIdListByOrganizeIdAndDepartmentId(organizeId, departmentId);

        // 用户所拥有的权限
        List<SysRefUserDepartment> refList = personnelService.findSysRefUserDepartmentByUserId(userId);
        if (refList != null && refList.size() > 0) {
            refList.forEach(ref -> refFilterIds.add(ref.getDepartmentId()));
        }

        if (refIdList.size() <= 0) {
            refIdList = refFilterIds;
        } else {
            // List 取交集
            refIdList = (ArrayList<Long>) CollectionUtils.intersection(refIdList, refFilterIds);
        }
        return refIdList.stream().distinct().collect(Collectors.toList());
    }

    @Override
    public List<Long> findDepartmentIdListByOrganizeIdAndDepartmentId(Long organizeId, Long departmentId) {
        List<Long> refIdList = new ArrayList<>();

        // 通过组织获取部门信息
        if (organizeId != null && organizeId > 0) {
            List<Long> organizeIdList = findSysOrganizeChildIdList(organizeId);
            List<SysDepartment> departmentList = personnelService.findSysDepartmentByOrganizeIdList(organizeIdList);
            if (departmentList != null && departmentList.size() > 0) {
                for (SysDepartment department : departmentList) {
                    refIdList.add(department.getId());
                }
            }
        }

        // 通过父节点查询部门信息
        if (departmentId != null && departmentId > 0) {
            List<Long> departmentIdList = findSysDepartmentChildIdList(departmentId);
            if (departmentIdList != null && departmentIdList.size() > 0) {
                refIdList.addAll(departmentIdList);
            }
        }

        return refIdList.stream().distinct().collect(Collectors.toList());
    }

    /********************************************* Start CommonTreeModel Tree *********************************************/

    /********************************************* Start SysDepartment Tree *********************************************/
    @Override
    public List<SysDepartment> packTreeSysDepartment(List<SysDepartment> list, Long parentId) {
        List<SysDepartment> result = new ArrayList<>();

        if (list != null && list.size() > 0) {
            List<SysDepartment> copyList = new ArrayList<>(list);
            list.forEach(department -> {
                if (department.getParentId() != null && department.getParentId().equals(parentId)) {
                    result.add(department);
                    copyList.remove(department);
                    department.setChildList(packTreeSysDepartment(copyList, department.getId()));
                }
            });
        }

        return result;
    }

    @Override
    public List<SysDepartment> packTreeSysDepartmentBySysRefUserOrganize(List<SysDepartment> list, List<SysRefUserDepartment> refList) {
        List<Long> ids = new ArrayList<>(refList.size());

        if (refList != null && refList.size() > 0) {
            refList.forEach(ref -> ids.add(ref.getDepartmentId()));
        }

        return packTreeSysDepartmentByIds(list, ids);
    }

    @Override
    public List<SysDepartment> packTreeSysDepartmentBySysRefUserInfoOrganize(List<SysDepartment> list, List<SysRefUserInfoDepartment> refList) {
        List<Long> ids = new ArrayList<>(refList.size());

        if (refList != null && refList.size() > 0) {
            refList.forEach(ref -> ids.add(ref.getDepartmentId()));
        }

        return packTreeSysDepartmentByIds(list, ids);
    }

    @Override
    public List<SysDepartment> packTreeSysDepartmentByIds(List<SysDepartment> list, List<Long> ids) {
        List<SysDepartment> result = new ArrayList<>();

        if (list != null && list.size() > 0) {
            list.forEach(department -> {
                if (department.getChildList() == null || department.getChildList().size() <= 0) {
                    if (ids.contains(department.getId())) {
                        result.add(department);
                    }
                } else {
                    List<SysDepartment> childList = packTreeSysDepartmentByIds(department.getChildList(), ids);
                    if (ids.contains(department.getId()) || childList != null && childList.size() > 0) {
                        department.setChildList(childList);
                        result.add(department);
                    }
                }
            });
        }

        return result;
    }

    @Override
    public SysDepartment packTreeSysDepartmentById(List<SysDepartment> list, Long id) {
        if (list != null && list.size() > 0) {
            for (SysDepartment department : list) {
                if (department.getId().equals(id)) {
                    department.setChildList(packTreeSysDepartment(list, department.getId()));
                    return department;
                }
            }
        }
        return null;
    }

    @Override
    public List<Long> findChildSysDepartment(SysDepartment department) {
        List<Long> result = new ArrayList<>();

        if (department != null && department.getChildList() != null && department.getChildList().size() > 0) {
            for (SysDepartment dep : department.getChildList()) {
                result.add(dep.getId());
                result.addAll(findChildSysDepartment(dep));
            }
        }

        return result;
    }

    @Override
    public List<Long> findSysDepartmentChildIdList(Long id) {
        SysDepartment department = departmentService.get(id);
        if (department != null) {
            List<SysDepartment> list = departmentService.findAll();
            if (list != null && list.size() > 0) {
                SysDepartment treeDep = packTreeSysDepartmentById(list, department.getId());
                List<Long> idList = findChildSysDepartment(treeDep);
                idList.add(department.getId());
                return idList;
            }
        }
        return Collections.emptyList();
    }

    /********************************************* End SysDepartment Tree *********************************************/

    /********************************************* Start SysMenu Tree *********************************************/
    @Override
    public List<SysMenu> packTreeSysMenu(List<SysMenu> list, Long parentId) {
        List<SysMenu> result = new ArrayList<>();

        if (list != null && list.size() > 0) {
            List<SysMenu> copyList = new ArrayList<>(list);
            list.forEach(menu -> {
                if (menu.getParentId() != null && menu.getParentId().equals(parentId)) {
                    result.add(menu);
                    copyList.remove(menu);
                    menu.setChildList(packTreeSysMenu(copyList, menu.getId()));
                }
            });
        }

        return result;
    }

    @Override
    public List<SysMenu> packTreeSysMenuBySysRefRoleMenu(List<SysMenu> list, List<SysRefRoleMenu> refList) {
        List<Long> ids = new ArrayList<>(refList.size());

        if (refList != null && refList.size() > 0) {
            refList.forEach(ref -> ids.add(ref.getMenuId()));
        }

        return packTreeSysMenuByIds(list, ids);
    }

    @Override
    public List<SysMenu> packTreeSysMenuByIds(List<SysMenu> list, List<Long> ids) {
        List<SysMenu> result = new ArrayList<>();

        if (list != null && list.size() > 0) {
            list.forEach(menu -> {
                if (menu.getChildList() == null || menu.getChildList().size() <= 0) {
                    if (ids.contains(menu.getId())) {
                        result.add(menu);
                    }
                } else {
                    List<SysMenu> childList = packTreeSysMenuByIds(menu.getChildList(), ids);
                    if (ids.contains(menu.getId()) || childList != null && childList.size() > 0) {
                        menu.setChildList(childList);
                        result.add(menu);
                    }
                }
            });
        }

        return result;
    }

    @Override
    public SysMenu packTreeSysMenuById(List<SysMenu> list, Long id) {
        if (list != null && list.size() > 0) {
            for (SysMenu menu : list) {
                if (menu.getId().equals(id)) {
                    menu.setChildList(packTreeSysMenu(list, menu.getId()));
                    return menu;
                }
            }
        }
        return null;
    }

    @Override
    public List<Long> findChildSysMenu(SysMenu menu) {
        List<Long> result = new ArrayList<>();

        if (menu != null && menu.getChildList() != null && menu.getChildList().size() > 0) {
            for (SysMenu sp : menu.getChildList()) {
                result.add(sp.getId());
                result.addAll(findChildSysMenu(sp));
            }
        }

        return result;
    }

    @Override
    public List<SysMenu> packTreeSysMenuByUserId(Long userId) {
        return packTreeSysMenuByUserIdAndParentId(userId, CommonEnumUtil.PARENT.DEFAULT.getValue());
    }

    @Override
    public List<SysMenu> packTreeSysMenuByUserIdAndParentId(Long userId, Long parentId) {
        List<SysRefUserRole> roleList = personnelService.findSysRefUserRole(userId);

        if (roleList != null && roleList.size() > 0) {
            List<SysRefRoleMenu> refRoleMenuList = personnelService.findSysRefRoleMenuByRefUserRole(roleList);
            if (refRoleMenuList != null && refRoleMenuList.size() > 0) {
                List<SysMenu> menuList = menuService.findAll();

                List<SysMenu> menuRootTree = packTreeSysMenu(menuList, parentId);
                return packTreeSysMenuBySysRefRoleMenu(menuRootTree, refRoleMenuList);
            }
        }
        return Collections.emptyList();
    }

    @Override
    public List<SysMenu> packTreeSysMenuByParentIdAndMenuType(List<SysMenu> list, Long parentId, Integer menuType) {
        List<SysMenu> result = new ArrayList<>();

        if (list != null && list.size() > 0) {
            for (SysMenu menu : list) {
                if (menu.getChildList() != null && menu.getChildList().size() > 0) {
                    result.addAll(packTreeSysMenuByParentIdAndMenuType(menu.getChildList(), parentId, menuType));
                }
                if (menu.getParentId().equals(parentId) && menu.getMenuType().equals(menuType)) {
                    result.add(menu);
                }
            }
        }

        return result;
    }

    @Override
    public List<SysMenu> packTreeSysMenu() {
        List<SysMenu> menuList = menuService.findAll();
        return packTreeSysMenu(menuList, CommonEnumUtil.PARENT.DEFAULT.getValue());
    }

    /********************************************* End SysMenu Tree *********************************************/

}
