package cn.estsh.i3plus.core.apiservice.serviceimpl.busi;

import cn.estsh.i3plus.core.api.iservice.busi.ISysFileService;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.platform.common.tool.BeanCopyTool;
import cn.estsh.i3plus.platform.common.tool.FileTool;
import cn.estsh.i3plus.platform.common.tool.StringTool;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.common.PagerHelper;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.platform.bean.SysFile;
import cn.estsh.i3plus.pojo.platform.repository.SysDictionaryRepository;
import cn.estsh.i3plus.pojo.platform.repository.SysFileRepository;
import cn.estsh.i3plus.pojo.platform.sqlpack.CoreHqlPack;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.fileservice.ImppFileService;
import cn.estsh.i3plus.pojo.base.bean.ImppFile;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.util.List;

/**
 * @Description : 文件资源服务接口
 * @Reference :
 * @Author : yunhao
 * @Date : 2018-10-25 16:28
 * @Modify :
 **/
@Service
public class SysFileServiceImpl implements ISysFileService {
    public static final Logger LOGGER = LoggerFactory.getLogger(SysFileServiceImpl.class);

    @Autowired
    private SysFileRepository sysFileRDao;

    @Autowired
    private SysDictionaryRepository sysDictionaryRDao;

    @Autowired
    private ImppFileService fileService;

    @Override
    @ApiOperation(value = "新增资源信息")
    public SysFile insertSysFile(SysFile sysFile) {
        LOGGER.info("文件资源 SYS_FILE :{}", sysFile);
        if (StringUtils.isNotBlank(sysFile.getFileCode())) {
            SysFile fileCode = sysFileRDao.getByProperty("fileCode", sysFile.getFileCode());
            if (fileCode != null) {
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                        .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                        .setErrorDetail("文件代码已存在。")
                        .setErrorSolution("请更换文件代码")
                        .build();
            }
        }

        return sysFileRDao.save(sysFile);
    }

    @Override
    @ApiOperation(value = "更新资源信息")
    public void updateSysFile(SysFile sysFile) {
        LOGGER.info("文件资源 SYS_FILE :{}", sysFile);
        SysFile originSysFile = sysFileRDao.getById(sysFile.getId());
        if(originSysFile == null){
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION_DATA_NOT_EXIT.getCode())
                    .setErrorDetail(ImppExceptionEnum.VARIFY_EXCEPTION_DATA_NOT_EXIT.getDescription())
                    .setErrorSolution("请重新选择")
                    .build();
        }

        // 文件代码唯一校验
        if (StringUtils.isNotBlank(sysFile.getFileCode())) {
            List list = sysFileRDao.findByHqlWhere(CoreHqlPack.packHqlSysFileCode(sysFile));
            if (list != null && list.size() > 0) {
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                        .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                        .setErrorDetail("文件代码已存在。")
                        .setErrorSolution("请更换文件代码")
                        .build();
            }
        }

        BeanCopyTool.beanCopy(sysFile, originSysFile);

        sysFileRDao.update(originSysFile);
    }

    @Override
    @ApiOperation(value = "查询资源信息", notes = "查询所有资源信息")
    public List<SysFile> listSysFile() {
        LOGGER.info("文件资源 SYS_FILE list");
        return sysFileRDao.list();
    }

    @Override
    @ApiOperation(value = "查询资源信息", notes = "根据 ID查询资源信息")
    public SysFile getSysFileById(Long id) {
        LOGGER.info("文件资源 SYS_FILE id:{}", id);
        return sysFileRDao.getById(id);
    }

    @Override
    @ApiOperation(value = "查询资源信息", notes = "查询资源信息(分页、组合)")
    public ListPager querySysFileByPager(SysFile sysFile, Pager pager) {
        LOGGER.info("文件资源 SYS_FILE sysFile:{}，Pager:{}", sysFile, pager);
        if (sysFile == null) {
            pager = PagerHelper.getPager(pager, sysFileRDao.listCount());
            return new ListPager(sysFileRDao.listPager(pager), pager);
        } else {
            DdlPackBean hqlPack = CoreHqlPack.packHqlSysFile(sysFile);
            pager = PagerHelper.getPager(pager, sysFileRDao.findByHqlWhereCount(hqlPack));
            return new ListPager(sysFileRDao.findByHqlWherePage(hqlPack, pager), pager);
        }
    }

    @Override
    @ApiOperation(value = "删除资源信息", notes = "根据ID删除资源信息")
    public void deleteSysFileById(Long id) {
        LOGGER.info("文件资源 SYS_FILE id:{}", id);
        sysFileRDao.deleteById(id);
    }

    @Override
    public void deleteSysFileByIds(Long[] ids) {
        LOGGER.info("文件资源 SYS_FILE ids:{}", ids);
        sysFileRDao.deleteByIds(ids);
    }

    @Override
    @ApiOperation(value = "上传文件资源", notes = "上传文件资源")
    public SysFile uploadFile(Object file) throws Exception {
        SysFile sysFile = uploadFastDfs(file, null);
        return insertSysFile(sysFile);
    }

    @Override
    @ApiOperation(value = "上传文件资源", notes = "上传文件资源")
    public SysFile uploadFile(Object file,SysFile sysFile, String userName) throws Exception {
        sysFile = uploadFastDfs(file, sysFile);
        ConvertBean.serviceModelInitialize(sysFile, userName);

        return insertSysFile(sysFile);
    }

    /**
     * 上传文件至fastdfs
     * @param file
     * @param sysFile
     * @return
     * @throws Exception
     */
    private SysFile uploadFastDfs(Object file,SysFile sysFile) throws Exception {
        if(sysFile == null){
            sysFile = new SysFile();
        }
        // 设置文件默认所属模块
        if(sysFile.getFileSoftTypeVal() == 0){
            sysFile.setFileSoftType(CommonEnumUtil.SOFT_TYPE.CORE.getValue());
        }

        ImppFile fastDFSFile;
        SysFile uploadFile;
        // 判断参数类型
        if (file instanceof File) {
            fastDFSFile = new ImppFile(((File) file).getName(), FileTool.file2Byte(((File) file).getPath()),
                    StringTool.getStringFileSuffix(((File) file).getName(), true));
            uploadFile = fileService.upload(fastDFSFile, sysFile.getFileSoftTypeVal());
        } else {
            uploadFile = fileService.upload((MultipartFile) file, sysFile.getFileSoftTypeVal());
        }

        uploadFile.setId(sysFile.getId());
        // 设置文件代码
        uploadFile.setFileCode(sysFile.getFileCode());

        return uploadFile;
    }

    @Override
    public SysFile getSysFileByFileCode(String fileCode) {
        return sysFileRDao.getByProperty("fileCode", fileCode);
    }
}
