package cn.estsh.i3plus.core.apiservice.serviceimpl.busi;

import cn.estsh.i3plus.core.api.iservice.busi.ISysUserPasswordService;
import cn.estsh.i3plus.core.api.iservice.busi.ISysUserService;
import cn.estsh.i3plus.core.apiservice.util.SysUserPasswordUtil;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.platform.common.tool.TimeTool;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.jpa.dao.BaseRepository;
import cn.estsh.i3plus.pojo.platform.bean.SysUser;
import cn.estsh.i3plus.pojo.platform.bean.SysUserPassword;
import cn.estsh.i3plus.pojo.platform.repository.SysUserPasswordRepository;
import cn.estsh.impp.framework.base.service.CrudService;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Primary;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * @Description :
 * @Reference :
 * @Author : wei.peng
 * @CreateDate : 20-6-4 下午3:49
 * @Modify:
 **/
@Primary
@Service
public class SysUserSavePasswordService extends CrudService<SysUserPassword> implements ISysUserPasswordService {

    @Autowired
    private SysUserPasswordRepository userPasswordRDao;

    @Autowired
    private ISysUserService userService;

    @Autowired
    private SysUserPasswordUtil userPasswordUtil;

    @Override
    public BaseRepository<SysUserPassword, Long> getRepository() {
        return userPasswordRDao;
    }

    @Override
    public void updatePassword(Long userId, String password, String newPwd) {
        SysUser user = userService.getSysUserById(userId);
        ValidatorBean.checkNotNull(user,"不存在的用户信息");

        SysUserPassword userPassword = userPasswordRDao.getById(user.getUserLoginPasswordId());
        ValidatorBean.checkNotNull(userPassword,"不存在的用户密码信息");
        if(StringUtils.equals(userPassword.getUserPassword(),password)){
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                    .setErrorCode(ImppExceptionEnum.NOT_CONFIG_EXCEPTION.getCode())
                    .setErrorDetail("原始密码错误")
                    .setErrorSolution("请重新操作")
                    .build();
        }

        updatePassword(userId, newPwd);
    }

    @Override
    public void updatePassword(Long userId, String password) {
        SysUser user = userService.getSysUserById(userId);
        ValidatorBean.checkNotNull(user,"不存在的用户信息");
        user.setUserLoginPassword(password);
        userPasswordUtil.checkPasswordSave(user);

        userPasswordRDao.updateByProperties(
                new String[]{"userId","isDeleted"},
                new Object[]{user.getId(),CommonEnumUtil.IS_DEAL.NO.getValue()},
                new String[]{"isDeleted","modifyDatetime","modifyUser"},
                new Object[]{
                        CommonEnumUtil.IS_DEAL.YES.getValue(), TimeTool.getNowTime(true), AuthUtil.getSessionUser().getUserName()});

        SysUserPassword pwd = new SysUserPassword();
        pwd.setUserId(userId);
        pwd.setUserPassword(SysUserPasswordUtil.encoder(password));
        ConvertBean.serviceModelInitialize(pwd,AuthUtil.getSessionUser());
        SysUserPassword save = userPasswordRDao.save(pwd);

        user.setUserLoginPasswordId(save.getId());
        ConvertBean.serviceModelUpdate(user,AuthUtil.getSessionUser().getUserName());
        userService.updateSysUser(user);
    }
}
