package cn.estsh.i3plus.core.apiservice.util;

import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.platform.common.util.CommonConstWords;
import cn.estsh.i3plus.platform.common.util.PlatformConstWords;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.platform.bean.MailConfig;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ImppRedis;
import cn.estsh.impp.framework.boot.util.RedisCacheTool;
import com.sun.mail.smtp.SMTPAddressFailedException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;
import javax.annotation.Resource;
import javax.mail.Address;
import javax.mail.AuthenticationFailedException;
import javax.mail.Message;
import javax.mail.MessagingException;
import javax.mail.Session;
import javax.mail.Transport;
import javax.mail.internet.AddressException;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeMessage;
import javax.mail.internet.MimeUtility;
import java.io.UnsupportedEncodingException;
import java.util.Properties;

/**
 * @Description : 邮件工具类
 * @Reference :
 * @Author : yunhaoo
 * @CreateDate : 2018-11-14 19:24
 * @Modify:
 **/
@Component
public class MailUtil {
    public static final Logger LOGGER = LoggerFactory.getLogger(MailUtil.class);

    // 收件人
    private Address[] to = new Address[]{};
    private Address[] cc = new Address[]{};
    private int mailSwitch = CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue();
    private String from = "";
    private String nick = "";
    private String title = "";
    private String content = "";
    private String smtpHost = "";
    private int smtpPort = 25;
    private String contentType = "";

    private String smtpUser = "";
    private String smtpPassword = "";
    private boolean isAuthenticationSMTP = false;

    @Resource(name = CommonConstWords.IMPP_REDIS_RES)
    private ImppRedis redisRes;

    /**
     * 初始化服务器邮箱参数
     *
     * @return 初始化
     */
    public MailUtil init() {
        try {
            // 查询是否开启邮件服务器
            this.mailSwitch = RedisCacheTool.getSysConfigIntVal(PlatformConstWords.MAIL_SWITCH,
                    CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue());
            this.smtpHost = RedisCacheTool.getSysConfigStrVal(PlatformConstWords.MAIL_HOST);
            this.smtpPort = RedisCacheTool.getSysConfigIntVal(PlatformConstWords.MAIL_PORT);
            this.from = RedisCacheTool.getSysConfigStrVal(PlatformConstWords.MAIL_USER);
            this.nick = RedisCacheTool.getSysConfigStrVal(PlatformConstWords.MAIL_NICK);
            this.smtpUser = RedisCacheTool.getSysConfigStrVal(PlatformConstWords.MAIL_USER);
            this.smtpPassword = RedisCacheTool.getSysConfigStrVal(PlatformConstWords.MAIL_PASSWORD);
            this.isAuthenticationSMTP = true;
            return this;
        } catch (Exception e) {
            LOGGER.error("邮箱配置初始化失败！", e.getMessage());
            return new MailUtil();
        }

    }

    /**
     * 初始化服务器邮箱参数
     *
     * @param mailConfig 邮箱配置信息
     * @return 初始化
     */
    public MailUtil init(MailConfig mailConfig) {
        try {
            this.mailSwitch = mailConfig.getMailSwitch();
            this.smtpHost = mailConfig.getMailHost();
            this.smtpPort = mailConfig.getMailPort();
            this.from = mailConfig.getMailUser();
            this.nick = mailConfig.getMailNick();
            this.smtpUser = mailConfig.getMailUser();
            this.smtpPassword = mailConfig.getMailPassword();
            this.isAuthenticationSMTP = true;

            return this;
        } catch (Exception e) {
            LOGGER.error("邮箱配置初始化失败！", e.getMessage());
            return new MailUtil();
        }
    }

    /**
     * 设置收件人地址
     *
     * @param nick 收件人Email地址
     */
    public void setNick(String nick) {
        this.nick = nick;
    }


    /**
     * 设置收件人地址
     *
     * @param aEmail 收件人Email地址
     */
    public void setTo(String aEmail) {
        String[] s = new String[1];
        s[0] = aEmail;
        this.to = getAddress(s);
    }

    /**
     * 设置多个收件人地址
     *
     * @param emails 收件人Email地址
     */
    public void setTo(String[] emails) {
        this.to = getAddress(emails);
    }

    /**
     * 设置抄送地址
     *
     * @param aEmail 抄送地址
     */
    public void setCC(String aEmail) {
        String[] s = new String[1];
        s[0] = aEmail;
        this.cc = getAddress(s);
    }

    /**
     * 设置多个抄送地址
     *
     * @param emails 抄送地址
     */
    public void setCC(String[] emails) {
        this.cc = getAddress(emails);
    }

    /**
     * 设置邮件主题
     *
     * @param mailTitle 邮件主题
     */
    public void setSubject(String mailTitle) {
        this.title = mailTitle;
    }

    /**
     * 设置邮件文字内容
     *
     * @param mailContent 邮件文字内容
     */
    public void setBody(String mailContent) {
        this.content = mailContent;
    }

    /**
     * 设置邮件字符类型
     *
     * @param contentType 请从CommonConstWords.MAIL_MODE_TEXT和MAIL_MODE_HTML中选择
     */
    public void setContentType(String contentType) {
        this.contentType = contentType;
    }

    /**
     * 获取邮箱地址
     *
     * @param add 邮箱地址字符串
     * @return
     */
    private Address[] getAddress(String[] add) {
        Address[] a = new Address[add.length];
        for (int i = 0; i < add.length; i++) {
            try {
                a[i] = new InternetAddress(add[i]);
            } catch (AddressException ex) {
                LOGGER.error("【" + add[i] + "】邮箱地址错误！", ex.getMessage());
            }
        }
        return a;
    }

    /**
     * 发送邮件
     */
    public void send() {
        try {
            // 是否开启邮箱配置
            if (this.mailSwitch != CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue()) {
                return;
            }
            Properties server = new Properties();
            server.put("mail.smtp.port", String.valueOf(this.smtpPort));
            server.put("mail.smtp.host", this.smtpHost);
            if (this.isAuthenticationSMTP) {
                server.put("mail.smtp.auth", "true");
            }

            Session conn = Session.getInstance(server, null);

            MimeMessage msg = new MimeMessage(conn);
            if (nick != null && !"".equals(nick)) {
                nick = MimeUtility.encodeText(nick, MimeUtility.mimeCharset("gb2312"), null);
                msg.setSubject(nick);
                msg.setFrom(new InternetAddress(nick + " <" + from + ">"));
            } else {
                msg.setFrom(new InternetAddress(this.from));
            }
            // 收件人
            if (this.to != null) {
                msg.setRecipients(Message.RecipientType.TO, this.to);
            }
            // 抄送
            if (this.cc != null) {
                msg.setRecipients(Message.RecipientType.CC, this.cc);
            }

            msg.setSubject(MimeUtility.encodeText(this.title, MimeUtility.mimeCharset("gb2312"), null));
            msg.setSubject(this.title);

            // 消息主体内容 处理
            this.content = this.content == null ? "" : this.content;

            msg.setContent(this.content, this.contentType);

            msg.saveChanges();
            if (this.isAuthenticationSMTP) {
                Transport transport = conn.getTransport("smtp");
                transport.connect(this.smtpHost, this.smtpUser, this.smtpPassword);
                transport.sendMessage(msg, msg.getAllRecipients());
                transport.close();
            } else {
                Transport.send(msg, msg.getAllRecipients());
            }
            LOGGER.info("邮件发送成功");

        } catch (SMTPAddressFailedException e) {
            LOGGER.error(e.getClass() + "\t" + e.getMessage());
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.IMPP.getCode())
                    .setErrorCode(ImppExceptionEnum.MAIL_RECIPIENT_NOT_EXIST.getCode())
                    .setErrorDetail(ImppExceptionEnum.MAIL_RECIPIENT_NOT_EXIST.getDescription())
                    .setErrorSolution("请重新设置收件人")
                    .build();
        } catch (AuthenticationFailedException e) {
            LOGGER.error(e.getClass() + "\t" + e.getMessage());
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.IMPP.getCode())
                    .setErrorCode(ImppExceptionEnum.MAIL_LOGIN_PASS_ERROR.getCode())
                    .setErrorDetail(ImppExceptionEnum.MAIL_LOGIN_PASS_ERROR.getDescription())
                    .setErrorSolution("请重新设置账号或密码")
                    .build();
        } catch (MessagingException e) {
            LOGGER.error(e.getClass() + "\t" + e.getMessage());
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.IMPP.getCode())
                    .setErrorCode(ImppExceptionEnum.MAIL_SERVER_CONFIG_ERROR.getCode())
                    .setErrorDetail(ImppExceptionEnum.MAIL_SERVER_CONFIG_ERROR.getDescription())
                    .setErrorSolution("请重新设置邮件服务器地址或端口")
                    .build();
        } catch (UnsupportedEncodingException e) {
            LOGGER.error(e.getClass() + "\t" + e.getMessage());
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.IMPP.getCode())
                    .setErrorCode(ImppExceptionEnum.MAIL_SERVER_CONFIG_ERROR.getCode())
                    .setErrorDetail(ImppExceptionEnum.MAIL_SERVER_CONFIG_ERROR.getDescription())
                    .setErrorSolution("请重新设置邮件服务器地址或端口")
                    .build();
        } catch (Exception e) {
            LOGGER.error(e.getClass() + "\t" + e.getMessage());
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.IMPP.getCode())
                    .setErrorCode(ImppExceptionEnum.MAIL_SERVER_CONFIG_ERROR.getCode())
                    .setErrorDetail(ImppExceptionEnum.MAIL_SERVER_CONFIG_ERROR.getDescription())
                    .setErrorSolution("邮件发送异常：%s", e.getMessage())
                    .build();
        }
    }
}
